@@ -27,6 +27,37 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323caps.h,v $
+ * Revision 1.39  2003/06/06 02:13:10  rjongbloed
+ * Changed non-standard capability semantics so can use C style strings as
+ *   the embedded data block (ie automatically call strlen)
+ *
+ * Revision 1.38  2003/04/28 07:00:00  robertj
+ * Fixed problem with compiler(s) not correctly initialising static globals
+ *
+ * Revision 1.37  2003/04/27 23:49:21  craigs
+ * Fixed some comments and made list of registered codecs
+ * available outside h323caps.cxx
+ *
+ * Revision 1.36  2002/11/09 04:24:01  robertj
+ * Fixed minor documentation errors.
+ *
+ * Revision 1.35  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.34  2002/09/03 06:19:36  robertj
+ * Normalised the multi-include header prevention ifdef/define symbol.
+ *
+ * Revision 1.33  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.32  2002/05/29 03:55:17  robertj
+ * Added protocol version number checking infrastructure, primarily to improve
+ *   interoperability with stacks that are unforgiving of new features.
+ *
+ * Revision 1.31  2002/05/10 05:44:50  robertj
+ * Added the max bit rate field to the data channel capability class.
+ *
  * Revision 1.30  2002/01/22 06:25:02  robertj
  * Moved payload type to ancestor so any capability can adjust it on logical channel.
  *
@@ -127,8 +158,13 @@
  *
  */
 
-#ifndef __H323CAPS_H
-#define __H323CAPS_H
+#ifndef __OPAL_H323CAPS_H
+#define __OPAL_H323CAPS_H
+
+#ifdef P_USE_PRAGMA
+#pragma interface
+#endif
+
 
 #include "codecs.h"
 #include "channels.h"
@@ -157,10 +193,6 @@ class H323Connection;
 class H323Capabilities;
 
 
-#ifdef __GNUC__
-#pragma interface
-#endif
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -365,6 +397,17 @@ class H323Capability : public PObject
     virtual BOOL IsNonStandardMatch(
       const H245_NonStandardParameter & param  /// Non standard field in PDU received
     ) const;
+
+    /**Validate that the capability is usable given the connection.
+       This checks agains the negotiated protocol version number and remote
+       application to determine if this capability should be used in TCS or
+       OLC pdus.
+
+       The default behaviour returns TRUE.
+      */
+    virtual BOOL IsUsable(
+      const H323Connection & connection
+    ) const;
   //@}
 
   /**@name Member variable access */
@@ -427,7 +470,7 @@ class H323NonStandardCapabilityInfo
     H323NonStandardCapabilityInfo(
       H323EndPoint & endpoint,        /// Endpoint to get t35 information
       const BYTE * dataBlock,         /// Non-Standard data for codec type
-      PINDEX dataSize,                /// Size of dataBlock
+      PINDEX dataSize,                /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -437,7 +480,7 @@ class H323NonStandardCapabilityInfo
     H323NonStandardCapabilityInfo(
       const PString & oid,
       const BYTE * dataBlock,         /// Non-Standard data for codec type
-      PINDEX dataSize,                /// Size of dataBlock
+      PINDEX dataSize,                /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,
       PINDEX comparisonLength = P_MAX_INDEX
     );
@@ -449,7 +492,7 @@ class H323NonStandardCapabilityInfo
       BYTE extension,                /// t35 information
       WORD maufacturer,              /// t35 information
       const BYTE * dataBlock,         /// Non-Standard data for codec type
-      PINDEX dataSize,                /// Size of dataBlock
+      PINDEX dataSize,                /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -547,8 +590,8 @@ class H323AudioCapability : public H323RealTimeCapability
     /**Create an audio based capability.
       */
     H323AudioCapability(
-      unsigned rxPacketSize, /// Maximum size of an audio packet in bytes
-      unsigned txPacketSize  /// Desired transmit size of an audio packet
+      unsigned rxPacketSize, /// Maximum size of an audio packet in frames
+      unsigned txPacketSize  /// Desired transmit size of an audio packet frames
     );
   //@}
 
@@ -729,11 +772,11 @@ class H323NonStandardAudioCapability : public H323AudioCapability,
     /**Create a new set of information about a non-standard codec.
       */
     H323NonStandardAudioCapability(
-      unsigned maxPacketSize,         /// Maximum size of an audio packet in bytes
-      unsigned desiredPacketSize,     /// Desired transmit size of an audio packet
+      unsigned maxPacketSize,         /// Maximum size of an audio packet in frames
+      unsigned desiredPacketSize,     /// Desired transmit size of an audio packet in frames
       H323EndPoint & endpoint,        /// Endpoint to get t35 information
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
      );
@@ -741,11 +784,11 @@ class H323NonStandardAudioCapability : public H323AudioCapability,
     /**Create a new set of information about a non-standard codec.
       */
     H323NonStandardAudioCapability(
-      unsigned maxPacketSize,         /// Maximum size of an audio packet in bytes
-      unsigned desiredPacketSize,     /// Desired transmit size of an audio packet
+      unsigned maxPacketSize,         /// Maximum size of an audio packet in frames
+      unsigned desiredPacketSize,     /// Desired transmit size of an audio packet in frames
       const PString & oid,            /// OID for indentification of codec
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -753,13 +796,13 @@ class H323NonStandardAudioCapability : public H323AudioCapability,
     /**Create a new set of information about a non-standard codec.
       */
     H323NonStandardAudioCapability(
-      unsigned maxPacketSize,         /// Maximum size of an audio packet in bytes
-      unsigned desiredPacketSize,     /// Desired transmit size of an audio packet
+      unsigned maxPacketSize,         /// Maximum size of an audio packet in frames
+      unsigned desiredPacketSize,     /// Desired transmit size of an audio packet in frames
       BYTE country,                   /// t35 information
       BYTE extension,                 /// t35 information
       WORD maufacturer,               /// t35 information
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -984,7 +1027,7 @@ class H323NonStandardVideoCapability : public H323VideoCapability,
     H323NonStandardVideoCapability(
       H323EndPoint & endpoint,        /// Endpoint to get t35 information
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -994,7 +1037,7 @@ class H323NonStandardVideoCapability : public H323VideoCapability,
     H323NonStandardVideoCapability(
       const PString & oid,            /// OID for indentification of codec
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -1006,7 +1049,7 @@ class H323NonStandardVideoCapability : public H323VideoCapability,
       BYTE extension,                 /// t35 information
       WORD maufacturer,               /// t35 information
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -1089,6 +1132,15 @@ class H323DataCapability : public H323Capability
   PCLASSINFO(H323DataCapability, H323Capability);
 
   public:
+  /**@name Construction */
+  //@{
+    /**Create a new data capability.
+      */
+    H323DataCapability(
+      unsigned maxBitRate = 0  /// Maximum bit rate for data in 100's b/s
+    );
+  //@}
+
   /**@name Identification functions */
   //@{
     /**Get the main type of the capability.
@@ -1213,6 +1265,9 @@ class H323DataCapability : public H323Capability
       const H245_DataApplicationCapability & pdu  /// PDU to set information on
     ) = 0;
   //@}
+
+  protected:
+    unsigned maxBitRate;
 };
 
 
@@ -1234,9 +1289,10 @@ class H323NonStandardDataCapability : public H323DataCapability,
     /**Create a new set of information about a non-standard codec.
       */
     H323NonStandardDataCapability(
+      unsigned maxBitRate,            /// Maximum bit rate for data in 100's b/s
       H323EndPoint & endpoint,        /// Endpoint to get t35 information
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -1244,9 +1300,10 @@ class H323NonStandardDataCapability : public H323DataCapability,
     /**Create a new set of information about a non-standard codec.
       */
     H323NonStandardDataCapability(
+      unsigned maxBitRate,            /// Maximum bit rate for data in 100's b/s
       const PString & oid,            /// OID for indentification of codec
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -1254,11 +1311,12 @@ class H323NonStandardDataCapability : public H323DataCapability,
     /**Create a new set of information about a non-standard codec.
       */
     H323NonStandardDataCapability(
+      unsigned maxBitRate,            /// Maximum bit rate for data in 100's b/s
       BYTE country,                   /// t35 information
       BYTE extension,                 /// t35 information
       WORD maufacturer,               /// t35 information
       const BYTE * dataBlock = NULL,  /// Non-Standard data for codec type
-      PINDEX dataSize = 0,            /// Size of dataBlock
+      PINDEX dataSize = 0,            /// Size of dataBlock. If 0 and dataBlock != NULL use strlen(dataBlock)
       PINDEX comparisonOffset = 0,    /// Offset into dataBlock to compare
       PINDEX comparisonLength = P_MAX_INDEX  /// Length of bytes to compare
     );
@@ -1333,7 +1391,7 @@ class H323NonStandardDataCapability : public H323DataCapability,
 ///////////////////////////////////////////////////////////////////////////////
 // The simplest codec is the G.711 PCM codec.
 
-/**This class describes the G711 codec capability.
+/**This class describes the G.711 codec capability.
  */
 class H323_G711Capability : public H323AudioCapability
 {
@@ -1541,6 +1599,18 @@ class H323_UserInputCapability : public H323Capability
       const H245_DataType & pdu,  /// PDU to get information from
       BOOL receiver               /// Is receiver OLC
     );
+
+    /**Validate that the capability is usable given the connection.
+       This checks agains the negotiated protocol version number and remote
+       application to determine if this capability should be used in TCS or
+       OLC pdus.
+
+       The default behaviour will check for early versions and return FALSE
+       for RFC2833 mode.
+      */
+    virtual BOOL IsUsable(
+      const H323Connection & connection
+    ) const;
   //@}
 
     static void AddAllCapabilities(
@@ -1617,7 +1687,7 @@ class H323Capabilities : public PObject
     );
   //@}
 
-  /**@name OVerrides from class PObject */
+  /**@name Overrides from class PObject */
   //@{
     /**Print out the object to the stream, virtual version of << operator.
      */
@@ -1825,7 +1895,8 @@ class H323Capabilities : public PObject
     /**Build a H.245 PDU from the information in the capability set.
       */
     void BuildPDU(
-      H245_TerminalCapabilitySet & pdu
+      const H323Connection & connection,  /// Connection building PDU for
+      H245_TerminalCapabilitySet & pdu    /// PDU to build
     ) const;
 
     /**Merge the capabilities into this set.
@@ -1891,13 +1962,21 @@ class H323CapabilityRegistration : public PCaselessString
       const char * name
     );
     virtual H323Capability * Create(H323EndPoint & ep) const = 0;
+
   protected:
+    static PMutex & GetMutex();
+    static H323CapabilityRegistration * registeredCapabilitiesListHead;
+
     H323CapabilityRegistration * link;
+
   friend class H323Capability;
   friend class H323Capabilities;
+  friend class OpalDynaCodecDLL;
 };
 
 
+
+
 #define H323_REGISTER_CAPABILITY_FUNCTION(cls, name, epvar) \
 class cls##_Registration : public H323CapabilityRegistration { \
   public: \
@@ -1923,7 +2002,7 @@ H323Capability * cls##_Registration::Create(H323EndPoint & epvar) const
   static cls##_Registration * cls##_Registration_Static_Library_Loader = &cls##_Registration_Instance
 
 
-#endif // __H323CAPS_H
+#endif // __OPAL_H323CAPS_H
 
 
 /////////////////////////////////////////////////////////////////////////////
