@@ -27,6 +27,111 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323con.h,v $
+ * Revision 1.60  2003/04/30 00:28:50  robertj
+ * Redesigned the alternate credentials in ARQ system as old implementation
+ *   was fraught with concurrency issues, most importantly it can cause false
+ *   detection of replay attacks taking out an endpoint completely.
+ *
+ * Revision 1.59  2003/02/12 23:59:22  robertj
+ * Fixed adding missing endpoint identifer in SETUP packet when gatekeeper
+ * routed, pointed out by Stefan Klein
+ * Also fixed correct rutrn of gk routing in IRR packet.
+ *
+ * Revision 1.58  2002/11/27 06:54:52  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.57  2002/11/15 05:17:22  robertj
+ * Added facility redirect support without changing the call token for access
+ *   to the call. If it gets redirected a new H323Connection object is
+ *   created but it looks like the same thing to an application.
+ *
+ * Revision 1.56  2002/11/13 04:37:23  robertj
+ * Added ability to get (and set) Q.931 release complete cause codes.
+ *
+ * Revision 1.55  2002/11/10 06:17:26  robertj
+ * Fixed minor documentation errors.
+ *
+ * Revision 1.54  2002/11/05 00:24:09  robertj
+ * Added function to determine if Q.931 CONNECT sent/received.
+ *
+ * Revision 1.53  2002/10/31 00:31:47  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.52  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.51  2002/09/03 06:19:36  robertj
+ * Normalised the multi-include header prevention ifdef/define symbol.
+ *
+ * Revision 1.50  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.49  2002/08/05 05:17:37  robertj
+ * Fairly major modifications to support different authentication credentials
+ *   in ARQ to the logged in ones on RRQ. For both client and server.
+ * Various other H.235 authentication bugs and anomalies fixed on the way.
+ *
+ * Revision 1.48  2002/07/05 02:22:56  robertj
+ * Added support for standard and non-standard T.38 mode change.
+ *
+ * Revision 1.47  2002/07/04 00:40:31  robertj
+ * More H.450.11 call intrusion implementation, thanks Aleksandar Todorovic
+ *
+ * Revision 1.46  2002/06/22 06:11:30  robertj
+ * Fixed bug on sometimes missing received endSession causing 10 second
+ *   timeout in connection clean up.
+ *
+ * Revision 1.45  2002/06/22 05:48:38  robertj
+ * Added partial implementation for H.450.11 Call Intrusion
+ *
+ * Revision 1.44  2002/06/13 06:15:19  robertj
+ * Allowed TransferCall() to be used on H323Connection as well as H323EndPoint.
+ *
+ * Revision 1.43  2002/06/05 08:58:58  robertj
+ * Fixed documentation of remote application name string.
+ * Added missing virtual keywards on some protocol handler functions.
+ *
+ * Revision 1.42  2002/05/29 06:40:29  robertj
+ * Changed sending of endSession/ReleaseComplete PDU's to occur immediately
+ *   on call clearance and not wait for background thread to do it.
+ * Stricter compliance by waiting for reply endSession before closing down.
+ *
+ * Revision 1.41  2002/05/29 03:55:17  robertj
+ * Added protocol version number checking infrastructure, primarily to improve
+ *   interoperability with stacks that are unforgiving of new features.
+ *
+ * Revision 1.40  2002/05/21 09:32:49  robertj
+ * Added ability to set multiple alias names ona  connection by connection
+ *   basis, defaults to endpoint list, thanks Artis Kugevics
+ *
+ * Revision 1.39  2002/05/15 23:59:33  robertj
+ * Added memory management of created T.38 and T.120 handlers.
+ * Improved documentation for use of T.38 and T.120 functions.
+ * Added ability to initiate a mode change for non-standard T.38
+ *
+ * Revision 1.38  2002/05/07 01:31:51  dereks
+ * Fix typo in documentation.
+ *
+ * Revision 1.37  2002/05/03 05:38:15  robertj
+ * Added Q.931 Keypad IE mechanism for user indications (DTMF).
+ *
+ * Revision 1.36  2002/05/02 07:56:24  robertj
+ * Added automatic clearing of call if no media (RTP data) is transferred in a
+ *   configurable (default 5 minutes) amount of time.
+ *
+ * Revision 1.35  2002/04/25 20:55:25  dereks
+ * Fix documentation. Thanks Olaf Schulz.
+ *
+ * Revision 1.34  2002/04/17 00:50:34  robertj
+ * Added ability to disable the in band DTMF detection.
+ *
  * Revision 1.33  2002/03/27 06:04:42  robertj
  * Added Temporary Failure end code for connection, an application may
  *   immediately retry the call if this occurs.
@@ -138,8 +243,13 @@
  *
  */
 
-#ifndef __H323CON_H
-#define __H323CON_H
+#ifndef __OPAL_H323CON_H
+#define __OPAL_H323CON_H
+
+#ifdef P_USE_PRAGMA
+#pragma interface
+#endif
+
 
 #include <ptclib/dtmf.h>
 #include "h323caps.h"
@@ -158,6 +268,8 @@ class PPER_Stream;
 class H225_EndpointType;
 class H225_TransportAddress;
 class H225_ArrayOf_PASN_OctetString;
+class H225_ProtocolIdentifier;
+class H225_AdmissionRequest;
 
 class H245_TerminalCapabilitySet;
 class H245_TerminalCapabilitySetReject;
@@ -180,6 +292,8 @@ class H245_JitterIndication;
 class H323SignalPDU;
 class H323ControlPDU;
 
+class H235Authenticators;
+
 class H245NegMasterSlaveDetermination;
 class H245NegTerminalCapabilitySet;
 class H245NegLogicalChannels;
@@ -190,6 +304,7 @@ class H450xDispatcher;
 class H4502Handler;
 class H4504Handler;
 class H4506Handler;
+class H45011Handler;
 
 class OpalT120Protocol;
 class OpalT38Protocol;
@@ -197,10 +312,6 @@ class OpalRFC2833;
 class OpalRFC2833Info;
 
 
-#ifdef __GNUC__
-#pragma interface
-#endif
-
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -218,17 +329,21 @@ class H323Connection : public PObject
   /**@name Construction */
   //@{
     enum Options {
-      FastStartOptionDisable      = 0x0001,
-      FastStartOptionEnable       = 0x0002,
-      FastStartOptionMask         = 0x0003,
+      FastStartOptionDisable       = 0x0001,
+      FastStartOptionEnable        = 0x0002,
+      FastStartOptionMask          = 0x0003,
 
-      H245TunnelingOptionDisable  = 0x0004,
-      H245TunnelingOptionEnable   = 0x0008,
-      H245TunnelingOptionMask     = 0x000c,
+      H245TunnelingOptionDisable   = 0x0004,
+      H245TunnelingOptionEnable    = 0x0008,
+      H245TunnelingOptionMask      = 0x000c,
 
-      H245inSetupOptionDisable    = 0x0010,
-      H245inSetupOptionEnable     = 0x0020,
-      H245inSetupOptionMask       = 0x0030,
+      H245inSetupOptionDisable     = 0x0010,
+      H245inSetupOptionEnable      = 0x0020,
+      H245inSetupOptionMask        = 0x0030,
+
+      DetectInBandDTMFOptionDisable= 0x0040,
+      DetectInBandDTMFOptionEnable = 0x0080,
+      DetectInBandDTMFOptionMask   = 0x00c0,
     };
 
     /**Create a new connection.
@@ -275,15 +390,23 @@ class H323Connection : public PObject
     void Unlock();
 
     /** Called when a connection is established.
-        Defautl behaviour is to call H323Connection::OnConnectionEstablished
+        Default behaviour is to call H323EndPoint::OnConnectionEstablished
       */
     virtual void OnEstablished();
 
     /** Called when a connection is cleared, just after CleanUpOnCallEnd()
-        Default behaviour is to call H323Connection::OnConnectionCleared
+        Default behaviour is to call H323EndPoint::OnConnectionCleared
       */
     virtual void OnCleared();
 
+    /**Determine if the call has been connected.
+       This indicates that Q.931 CONNECT has occurred. This usually means in
+       PSTN gateway environments that a charge will be made for the call. This
+       is not quite the same as IsEstablished() as that indicates the call is
+       connected AND there is media open.
+      */
+    BOOL IsConnected() const { return connectionState == HasExecutedSignalConnect || connectionState == EstablishedConnection; }
+
     /**Determine if the call has been established.
        This can be used in combination with the GetCallEndReason() function
        to determine the three main phases of a call, call setup, call
@@ -291,13 +414,6 @@ class H323Connection : public PObject
       */
     BOOL IsEstablished() const { return connectionState == EstablishedConnection; }
 
-    /**Inernal function to check if call established.
-       This checks all the criteria for establishing a call an initiating the
-       starting of media channels, if they have not already been started vi
-       the fast start algorithm.
-    */
-    virtual void InternalEstablishedConnectionCheck();
-
     /**Call clearance reasons.
        NOTE: if anything is added to this, you also need to add the field to
        the tables in h323.cxx and h323pdu.cxx.
@@ -326,6 +442,8 @@ class H323Connection : public PObject
       EndedByNoEndPoint,        /// The remote party is not running an endpoint
       EndedByHostOffline,       /// The remote party host off line
       EndedByTemporaryFailure,  /// The remote failed temporarily app may retry
+      EndedByQ931Cause,         /// The remote ended the call with unmapped Q.931 cause code
+      EndedByDurationLimit,     /// Call cleared due to an enforced duration limit
       NumCallEndReasons
     };
 #if PTRACING
@@ -346,7 +464,7 @@ class H323Connection : public PObject
        An application should have no cause to use this function. It is present
        for the H323EndPoint::ClearCall() function to set the clearance reason.
       */
-    void SetCallEndReason(
+    virtual void SetCallEndReason(
       CallEndReason reason,     /// Reason for clearance of connection.
       PSyncPoint * sync = NULL  /// syncpoint to use for synchronous destruction
     );
@@ -384,6 +502,7 @@ class H323Connection : public PObject
     /**Attach a transport to this connection as the signalling channel.
       */
     void AttachSignalChannel(
+      const PString & token,    /// New token to use to identify connection
       H323Transport * channel,  /// Transport for the PDU's
       BOOL answeringCall        /// Flag for if incoming/outgoing call.
     );
@@ -397,7 +516,7 @@ class H323Connection : public PObject
     /**Handle reading PDU's from the signalling channel.
        This is an internal function and is unlikely to be used by applications.
      */
-    void HandleSignallingChannel();
+    virtual void HandleSignallingChannel();
 
     /**Handle PDU from the signalling channel.
        This is an internal function and is unlikely to be used by applications.
@@ -573,7 +692,7 @@ class H323Connection : public PObject
 
        The return value is TRUE if the call is to be forwarded, FALSE
        otherwise. Note that if the call is forwarded the current connection is
-       cleared with teh ended call code of EndedByCallForwarded.
+       cleared with the ended call code of EndedByCallForwarded.
       */
     virtual BOOL ForwardCall(
       const PString & forwardParty   /// Party to forward call to.
@@ -585,7 +704,8 @@ class H323Connection : public PObject
      */
     void TransferCall(
       const PString & remoteParty,   /// Remote party to transfer the existing call to
-      const PString & callIdentity   /// Call Identity of secondary call if present
+      const PString & callIdentity = PString::Empty()
+                                    /// Call Identity of secondary call if present
     );
 
     /**Transfer the call through consultation so the remote party in the primary call is connected to
@@ -602,7 +722,7 @@ class H323Connection : public PObject
        This is an internal function and it is not expected the user will call
        it directly.
      */
-    void HandleConsultationTransfer(
+    virtual void HandleConsultationTransfer(
       const PString & callIdentity, /**Call Identity of secondary call 
                                        received in SETUP Message. */
       H323Connection & incoming     /// Connection upon which SETUP PDU was received.
@@ -620,7 +740,7 @@ class H323Connection : public PObject
        This is an internal function and it is not expected the user will call
        it directly.
      */
-    void HandleTransferCall(
+    virtual void HandleTransferCall(
       const PString & token,
       const PString & identity
     );
@@ -638,7 +758,7 @@ class H323Connection : public PObject
        Reception of a callTransferSetup return error APDU.
        Expiry of Call Transfer timer CT-T4.
      */
-    void HandleCallTransferFailure(
+    virtual void HandleCallTransferFailure(
       const int returnError    /// Failure reason code
     );
 
@@ -699,6 +819,50 @@ class H323Connection : public PObject
       */
     BOOL IsCallOnHold() const;
 
+    /**Begin a call intrusion request.
+       Calls h45011handler->IntrudeCall where SS pdu is added to Call Setup
+       message.
+      */
+    virtual void IntrudeCall(
+      unsigned capabilityLevel
+    );
+
+    /**Handle an incoming call instrusion request.
+       Calls h45011handler->AwaitSetupResponse where we set Handler state to
+       CI-Wait-Ack
+      */
+    virtual void HandleIntrudeCall(
+      const PString & token,
+      const PString & identity
+    );
+
+    /**Set flag indicating call intrusion.
+       Used to set a flag when intrusion occurs and to determine if
+       connection is created for Call Intrusion. This flag is used when we
+       should decide whether to Answer the call or to Close it.
+      */
+    void SetCallIntrusion() { isCallIntrusion = TRUE; }
+
+    BOOL IsCallIntrusion() { return isCallIntrusion; }
+
+    /**Get Call Intrusion Protection Level of the local endpoint.
+      */
+    unsigned GetLocalCallIntrusionProtectionLevel() { return callIntrusionProtectionLevel; }
+
+    /**Get Call Intrusion Protection Level of other endpoints that we are in
+       connection with.
+      */
+    virtual BOOL GetRemoteCallIntrusionProtectionLevel(
+      const PString & callToken,
+      unsigned callIntrusionProtectionLevel
+    );
+
+    virtual void SetIntrusionImpending();
+
+    virtual void SetForcedReleaseAccepted();
+
+    virtual void SetIntrusionNotAuthorized();
+
     /**Send a Call Waiting indication message to the remote endpoint using
        H.450.6.  The second paramter is used to indicate to the calling user
        how many additional users are "camped on" the called user. A value of
@@ -1087,7 +1251,7 @@ class H323Connection : public PObject
        it is about to send the Capabilities Set to the remote endpoint. This
        gives the application an oppurtunity to alter the PDU to be sent.
 
-       The default behaviour will make "adjustments" for compatibiliry with
+       The default behaviour will make "adjustments" for compatibility with
        some broken remote endpoints.
      */
     virtual void OnSendCapabilitySet(
@@ -1188,6 +1352,7 @@ class H323Connection : public PObject
     );
 
     /**Select default logical channel for fast start.
+       Internal function, not for normal use.
       */
     virtual void SelectFastStartChannels(
       unsigned sessionID,   /// Session ID to find default logical channel.
@@ -1471,12 +1636,16 @@ class H323Connection : public PObject
   /**@name Indications */
   //@{
     enum SendUserInputModes {
+      SendUserInputAsQ931,
       SendUserInputAsString,
       SendUserInputAsTone,
       SendUserInputAsInlineRFC2833,
       SendUserInputAsSeparateRFC2833,  // Not implemented
       NumSendUserInputModes
     };
+#if PTRACING
+    friend ostream & operator<<(ostream & o, SendUserInputModes m);
+#endif
 
     /**Set the user input indication transmission mode.
       */
@@ -1486,6 +1655,13 @@ class H323Connection : public PObject
       */
     SendUserInputModes GetSendUserInputMode() const { return sendUserInputMode; }
 
+    /**Get the real user input indication transmission mode.
+       This will return the user input mode that will actually be used for
+       transmissions. It will be the value of GetSendUserInputMode() provided
+       the remote endpoint is capable of that mode.
+      */
+    SendUserInputModes GetRealSendUserInputMode() const;
+
     /**Send a user input indication to the remote endpoint.
        This is for sending arbitrary strings as user indications.
 
@@ -1562,6 +1738,16 @@ class H323Connection : public PObject
       int duration = 500  /// Duration of tone in milliseconds
     ) { SendUserInputTone('!', duration); }
 
+    /**Send a user input indication to the remote endpoint.
+       This is for sending arbitrary strings as user indications.
+
+       This always uses a Q.931 Keypad Information Element in a Information
+       pdu sending the entire string in one go.
+      */
+    virtual void SendUserInputIndicationQ931(
+      const PString & value                   /// String value of indication
+    );
+
     /**Send a user input indication to the remote endpoint.
        This is for sending arbitrary strings as user indications.
 
@@ -1734,21 +1920,49 @@ class H323Connection : public PObject
        This is called when the OpenLogicalChannel subsystem requires that
        a T.120 channel be established.
 
-       The default behavour returns H323Endpoint::CreateT120ProtocolHandler().
+       Note that if the application overrides this and returns a pointer to a
+       heap variable (using new) then it is the responsibility of the creator
+       to subsequently delete the object. The user of this function (the 
+       H323_T120Channel class) will not do so.
+
+       The default behavour returns H323Endpoint::CreateT120ProtocolHandler()
+       while keeping track of that variable for autmatic deletion.
       */
-    virtual OpalT120Protocol * CreateT120ProtocolHandler() const;
+    virtual OpalT120Protocol * CreateT120ProtocolHandler();
 
     /**Create an instance of the T.38 protocol handler.
        This is called when the OpenLogicalChannel subsystem requires that
        a T.38 fax channel be established.
 
-       The default behavour returns H323Endpoint::CreateT38ProtocolHandler().
+       Note that if the application overrides this and returns a pointer to a
+       heap variable (using new) then it is the responsibility of the creator
+       to subsequently delete the object. The user of this function (the 
+       H323_T38Channel class) will not do so.
+
+       The default behavour returns H323Endpoint::CreateT38ProtocolHandler()
+       while keeping track of that variable for autmatic deletion.
       */
-    virtual OpalT38Protocol * CreateT38ProtocolHandler() const;
+    virtual OpalT38Protocol * CreateT38ProtocolHandler();
 
     /**Request a mode change to T.38 data.
       */
-    virtual BOOL RequestModeChangeT38();
+    virtual BOOL RequestModeChangeT38(
+      const char * capabilityNames = "T.38\nT38FaxUDP"
+    );
+
+    /**Get separate H.235 authentication for the connection.
+       This allows an individual ARQ to override the authentical credentials
+       used in H.235 based RAS for this particular connection.
+
+       A return value of FALSE indicates to use the default credentials of the
+       endpoint, while TRUE indicates that new credentials are to be used.
+
+       The default behavour does nothing and returns FALSE.
+      */
+    virtual BOOL GetAdmissionRequestAuthentication(
+      const H225_AdmissionRequest & arq,  /// ARQ being constructed
+      H235Authenticators & authenticators /// New authenticators for ARQ
+    );
   //@}
 
   /**@name Member variable access */
@@ -1761,6 +1975,15 @@ class H323Connection : public PObject
      */
     BOOL HadAnsweredCall() const { return callAnswered; }
 
+    /**Determined if connection is gatekeeper routed.
+     */
+    BOOL IsGatekeeperRouted() const { return gatekeeperRouted; }
+
+    /**Get the Q.931 cause code (Q.850) that terminated this call.
+       See Q931::CauseValues for common values.
+     */
+    unsigned GetQ931Cause() const { return q931Cause; }
+
     /**Get the distinctive ring code for incoming call.
        This returns an integer from 0 to 7 that may indicate to an application
        that different ring cadences are to be used.
@@ -1797,7 +2020,9 @@ class H323Connection : public PObject
 
     /**Set the local name/alias from information in the PDU.
       */
-    void SetLocalPartyName(const PString & name) { localPartyName = name; }
+    void SetLocalPartyName(const PString & name);
+
+    const PStringList & GetLocalAliasNames() const { return localAliasNames; }
 
     /**Get the remote party name.
        This returns a string indicating the remote parties names and aliases.
@@ -1831,10 +2056,12 @@ class H323Connection : public PObject
        This information is obtained from the sourceInfo field of the H.225
        Setup PDU or the destinationInfo of the call proceeding or alerting
        PDU's. The general format of the string will be information extracted
-       from the VendorIdentifier field of the EndpointType. In partcular:
+       from the VendorIdentifier field of the EndpointType. In particular:
 
-          productId\tversionId\tt35CountryCode:t35Extension:manufacturerCode
+          productId <tab> versionId <tab> t35CountryCode/manufacturerCode
 
+       for example
+          "Equivalence OpenPhone\t1.4.2\t9/61"
       */
     const PString & GetRemoteApplication() const { return remoteApplication; }
 
@@ -1860,10 +2087,18 @@ class H323Connection : public PObject
       */
     const H323Transport * GetSignallingChannel() const { return signallingChannel; }
 
+    /**Get the signalling channel protocol version number.
+      */
+    unsigned GetSignallingVersion() const { return h225version; }
+
     /**Get the control channel being used (may return signalling channel).
       */
     const H323Transport & GetControlChannel() const;
 
+    /**Get the control channel protocol version number.
+      */
+    unsigned GetControlVersion() const { return h245version; }
+
     /**Get the time at which the connection was begun
       */
     PTime GetSetupUpTime() const { return setupTime; }
@@ -1881,13 +2116,22 @@ class H323Connection : public PObject
       */
     PTime GetConnectionEndTime() const { return callEndTime; }
 
-    /**Get the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio jitter delay parameter.
+       Defaults to 50ms
      */
-    WORD GetMaxAudioDelayJitter() const { return maxAudioDelayJitter; }
+    unsigned GetMinAudioJitterDelay() const { return minAudioJitterDelay; }
 
-    /**Set the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio delay jitter parameter.
+       Defaults to 250ms.
      */
-    void SetMaxAudioDelayJitter(unsigned jitter) { maxAudioDelayJitter = (WORD)jitter; }
+    unsigned GetMaxAudioJitterDelay() const { return maxAudioJitterDelay; }
+
+    /**Set the maximum audio delay jitter parameter.
+     */
+    void SetAudioJitterDelay(
+      unsigned minDelay,   // New minimum jitter buffer delay in milliseconds
+      unsigned maxDelay    // New maximum jitter buffer delay in milliseconds
+    );
 
     /**Get the UUIE PDU monitor bit mask.
      */
@@ -1925,10 +2169,27 @@ class H323Connection : public PObject
        n - n plus this connection
      */
     const int GetRemoteCallWaiting() const { return remoteCallWaiting; }
+
+    /**Set the enforced duration limit for the call.
+       This starts a timer that will automatically shut down the call when it
+       expires.
+      */
+    void SetEnforcedDurationLimit(
+      unsigned seconds  /// max duration of call in seconds
+    );
   //@}
 
 
   protected:
+    /**Internal function to check if call established.
+       This checks all the criteria for establishing a call an initiating the
+       starting of media channels, if they have not already been started via
+       the fast start algorithm.
+    */
+    virtual void InternalEstablishedConnectionCheck();
+    BOOL InternalEndSessionCheck(PPER_Stream & strm);
+    void SetRemoteVersions(const H225_ProtocolIdentifier & id);
+    void MonitorCallStatus();
     PDECLARE_NOTIFIER(OpalRFC2833Info, H323Connection, OnUserInputInlineRFC2833);
     PDECLARE_NOTIFIER(H323Codec::FilterInfo, H323Connection, OnUserInputInBandDTMF);
 
@@ -1937,12 +2198,14 @@ class H323Connection : public PObject
 
     int                  remoteCallWaiting; // Number of call's waiting at the remote endpoint
     BOOL                 callAnswered;
+    BOOL                 gatekeeperRouted;
     unsigned             distinctiveRing;
     PString              callToken;
     unsigned             callReference;
     OpalGloballyUniqueID callIdentifier;
     OpalGloballyUniqueID conferenceIdentifier;
 
+    PStringList        localAliasNames;
     PString            localPartyName;
     H323Capabilities   localCapabilities; // Capabilities local system supports
     PString            remotePartyName;
@@ -1953,7 +2216,8 @@ class H323Connection : public PObject
     H323Capabilities   remoteCapabilities; // Capabilities remote system supports
     unsigned           remoteMaxAudioDelayJitter;
     PTimer             roundTripDelayTimer;
-    WORD               maxAudioDelayJitter;
+    unsigned           minAudioJitterDelay;
+    unsigned           maxAudioJitterDelay;
     unsigned           bandwidthAvailable;
     unsigned           uuiesRequested;
     PString            gkAccessTokenOID;
@@ -1986,21 +2250,34 @@ class H323Connection : public PObject
     PTime         connectedTime;
     PTime         callEndTime;
     CallEndReason callEndReason;
+    unsigned      q931Cause;
 
+    unsigned   h225version;
+    unsigned   h245version;
+    BOOL       h245versionSet;
     BOOL doH245inSETUP;
     BOOL lastPDUWasH245inSETUP;
+    BOOL detectInBandDTMF;
     BOOL mustSendDRQ;
     BOOL mediaWaitForConnect;
     BOOL transmitterSidePaused;
     BOOL earlyStart;
     BOOL startT120;
-    BOOL t38ModeChange;
+    PString    t38ModeChangeCapabilities;
     PSyncPoint digitsWaitFlag;
+    BOOL       endSessionNeeded;
+    BOOL       endSessionSent;
+    PSyncPoint endSessionReceived;
+    PTimer     enforcedDurationLimit;
 
     // Used as part of a local call hold operation involving MOH
     PChannel * holdMediaChannel;
     BOOL       isConsultationTransfer;
 
+    /** Call Intrusion flag and parameters */
+    BOOL     isCallIntrusion;
+    unsigned callIntrusionProtectionLevel;
+
     RTP_SessionManager rtpSessions;
 
     enum FastStartStates {
@@ -2032,7 +2309,11 @@ class H323Connection : public PObject
     H4502Handler                     * h4502handler;
     H4504Handler                     * h4504handler;
     H4506Handler                     * h4506handler;
+    H45011Handler                    * h45011handler;
     OpalRFC2833                      * rfc2833handler;
+    OpalT120Protocol                 * t120handler;
+    OpalT38Protocol                  * t38handler;
+
 
     // The In-Band DTMF detector. This is used inside an audio filter which is
     // added to the audio channel.
@@ -2050,7 +2331,8 @@ PLIST(H323ConnectionList, H323Connection);
 PDICTIONARY(H323ConnectionDict, PString, H323Connection);
 PDICTIONARY(H323CallIdentityDict, PString, H323Connection);
 
-#endif // __H323CON_H
+
+#endif // __OPAL_H323CON_H
 
 
 /////////////////////////////////////////////////////////////////////////////
