@@ -27,6 +27,126 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323ep.h,v $
+ * Revision 1.52  2003/04/24 01:49:33  dereks
+ * Add ability to set no media timeout interval
+ *
+ * Revision 1.51  2003/04/10 09:39:48  robertj
+ * Added associated transport to new GetInterfaceAddresses() function so
+ *   interfaces can be ordered according to active transport links. Improves
+ *   interoperability.
+ *
+ * Revision 1.50  2003/04/10 01:05:11  craigs
+ * Added functions to access to lists of interfaces
+ *
+ * Revision 1.49  2003/04/07 13:09:25  robertj
+ * Added ILS support to callto URL parsing in MakeCall(), ie can now call hosts
+ *   registered with an ILS directory.
+ *
+ * Revision 1.48  2003/02/13 00:11:31  robertj
+ * Added missing virtual for controlling call transfer, thanks Andrey Pinaev
+ *
+ * Revision 1.47  2003/02/09 00:48:06  robertj
+ * Added function to return if registered with gatekeeper.
+ *
+ * Revision 1.46  2003/02/04 07:06:41  robertj
+ * Added STUN support.
+ *
+ * Revision 1.45  2003/01/26 05:57:58  robertj
+ * Changed ParsePartyName so will accept addresses of the form
+ *   alias@gk:address which will do an LRQ call to "address" using "alias"
+ *   to determine the IP address to connect to.
+ *
+ * Revision 1.44  2003/01/23 02:36:30  robertj
+ * Increased (and made configurable) timeout for H.245 channel TCP connection.
+ *
+ * Revision 1.43  2002/11/28 01:19:55  craigs
+ * Added virtual to several functions
+ *
+ * Revision 1.42  2002/11/27 06:54:52  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.41  2002/11/15 05:17:22  robertj
+ * Added facility redirect support without changing the call token for access
+ *   to the call. If it gets redirected a new H323Connection object is
+ *   created but it looks like the same thing to an application.
+ *
+ * Revision 1.40  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.39  2002/10/31 00:32:15  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.38  2002/10/23 06:06:10  robertj
+ * Added function to be smarter in using a gatekeeper for use by endpoint.
+ *
+ * Revision 1.37  2002/10/21 06:07:44  robertj
+ * Added function to set gatekeeper access token OID.
+ *
+ * Revision 1.36  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.35  2002/09/10 06:32:25  robertj
+ * Added function to get gatekeeper password.
+ *
+ * Revision 1.34  2002/09/03 06:19:36  robertj
+ * Normalised the multi-include header prevention ifdef/define symbol.
+ *
+ * Revision 1.33  2002/07/19 03:39:19  robertj
+ * Bullet proofed setting of RTP IP port base, can't be zero!
+ *
+ * Revision 1.32  2002/07/18 01:50:10  robertj
+ * Changed port secltion code to force apps to use function interface.
+ *
+ * Revision 1.31  2002/06/22 05:48:38  robertj
+ * Added partial implementation for H.450.11 Call Intrusion
+ *
+ * Revision 1.30  2002/06/13 06:15:19  robertj
+ * Allowed TransferCall() to be used on H323Connection as well as H323EndPoint.
+ *
+ * Revision 1.29  2002/06/12 03:55:21  robertj
+ * Added function to add/remove multiple listeners in one go comparing against
+ *   what is already running so does not interrupt unchanged listeners.
+ *
+ * Revision 1.28  2002/05/29 06:40:29  robertj
+ * Changed sending of endSession/ReleaseComplete PDU's to occur immediately
+ *   on call clearance and not wait for background thread to do it.
+ * Stricter compliance by waiting for reply endSession before closing down.
+ *
+ * Revision 1.27  2002/05/28 06:15:09  robertj
+ * Split UDP (for RAS) from RTP port bases.
+ * Added current port variable so cycles around the port range specified which
+ *   fixes some wierd problems on some platforms, thanks Federico Pinna
+ *
+ * Revision 1.26  2002/05/17 03:38:05  robertj
+ * Fixed problems with H.235 authentication on RAS for server and client.
+ *
+ * Revision 1.25  2002/05/16 00:03:05  robertj
+ * Added function to get the tokens for all active calls.
+ * Improved documentation for use of T.38 and T.120 functions.
+ *
+ * Revision 1.24  2002/05/15 08:59:18  rogerh
+ * Update comments
+ *
+ * Revision 1.23  2002/05/03 05:38:15  robertj
+ * Added Q.931 Keypad IE mechanism for user indications (DTMF).
+ *
+ * Revision 1.22  2002/05/02 07:56:24  robertj
+ * Added automatic clearing of call if no media (RTP data) is transferred in a
+ *   configurable (default 5 minutes) amount of time.
+ *
+ * Revision 1.21  2002/04/18 01:41:07  robertj
+ * Fixed bad variable name for disabling DTMF detection, very confusing.
+ *
+ * Revision 1.20  2002/04/17 00:49:56  robertj
+ * Added ability to disable the in band DTMF detection.
+ *
  * Revision 1.19  2002/04/10 06:48:47  robertj
  * Added functions to set port member variables.
  *
@@ -91,11 +211,12 @@
  *
  */
 
-#ifndef __H323EP_H
-#define __H323EP_H
+#ifndef __OPAL_H323EP_H
+#define __OPAL_H323EP_H
 
 
 #include "h323con.h"
+#include "h235auth.h"
 
 
 /* The following classes have forward references to avoid including the VERY
@@ -106,15 +227,19 @@
 class H225_EndpointType;
 class H225_VendorIdentifier;
 class H225_H221NonStandard;
+class H225_ServiceControlDescriptor;
 
 class H323SignalPDU;
 class H323ConnectionsCleaner;
+class H323ServiceControlSession;
 
 class OpalT120Protocol;
 class OpalT38Protocol;
 
+class PSTUNClient;
 
-#ifdef __GNUC__
+
+#ifdef P_USE_PRAGMA
 #pragma interface
 #endif
 
@@ -138,6 +263,10 @@ class H323EndPoint : public PObject
   PCLASSINFO(H323EndPoint, PObject);
 
   public:
+    enum {
+      DefaultTcpPort = 1720
+    };
+
   /**@name Construction */
   //@{
     /**Create a new endpoint.
@@ -259,6 +388,29 @@ class H323EndPoint : public PObject
 
   /**@name Gatekeeper management */
   //@{
+    /**Use and register with an explicit gatekeeper.
+       This will call other functions according to the following table:
+
+           address    identifier   function
+           empty      empty        DiscoverGatekeeper()
+           non-empty  empty        SetGatekeeper()
+           empty      non-empty    LocateGatekeeper()
+           non-empty  non-empty    SetGatekeeperZone()
+
+       The localAddress field, if non-empty, indicates the interface on which
+       to look for the gatekeeper. An empty string is equivalent to "ip$*:*"
+       which is any interface or port.
+
+       If the endpoint is already registered with a gatekeeper that meets
+       the same criteria then the gatekeeper is not changed, otherwise it is
+       deleted (with unregistration) and new one created and registered to.
+     */
+    BOOL UseGatekeeper(
+      const PString & address = PString::Empty(),     /// Address of gatekeeper to use.
+      const PString & identifier = PString::Empty(),  /// Identifier of gatekeeper to use.
+      const PString & localAddress = PString::Empty() /// Local interface to use.
+    );
+
     /**Select and register with an explicit gatekeeper.
        This will use the specified transport and a string giving a transport
        dependent address to locate a specific gatekeeper. The endpoint will
@@ -335,6 +487,10 @@ class H323EndPoint : public PObject
      */
     H323Gatekeeper * GetGatekeeper() const { return gatekeeper; }
 
+    /**Return if endpoint is registered with gatekeeper.
+      */
+    BOOL IsRegisteredWithGatekeeper() const;
+
     /**Unregister and delete the gatekeeper we are registered with.
        The return value indicates FALSE if there was an error during the
        unregistration. However the gatekeeper is still removed and its
@@ -346,9 +502,17 @@ class H323EndPoint : public PObject
 
     /**Set the H.235 password for the gatekeeper.
       */
-    void SetGatekeeperPassword(
+    virtual void SetGatekeeperPassword(
       const PString & password
     );
+
+    /**Get the H.235 password for the gatekeeper.
+      */
+    virtual const PString & GetGatekeeperPassword() const { return gatekeeperPassword; }
+
+    /**Create a list of authenticators for gatekeeper.
+      */
+    virtual H235Authenticators CreateAuthenticators();
   //@}
 
   /**@name Connection management */
@@ -357,6 +521,15 @@ class H323EndPoint : public PObject
        This allows for the automatic creating of incoming call connections. An
        application should use OnConnectionEstablished() to monitor when calls
        have arrived and been successfully negotiated.
+
+       Note if this returns TRUE, then the endpoint is responsible for
+       deleting the H323Listener listener object. If FALSE is returned then
+       the object is not deleted and it is up to the caller to release the
+       memory allocated for the object.
+
+       If a listener already exists on the same transport address then it is
+       ignored, but TRUE is still returned. The caller does not need to delete
+       the object.
       */
     BOOL StartListener(
       H323Listener * listener /// Transport dependent listener.
@@ -366,11 +539,37 @@ class H323EndPoint : public PObject
        This allows for the automatic creating of incoming call connections. An
        application should use OnConnectionEstablished() to monitor when calls
        have arrived and been successfully negotiated.
+
+       If a listener already exists on the same address then it is ignored,
+       but TRUE is still returned.
+
+       If the iface string is empty then "*" is assumed which will listen on
+       the standard TCP port on INADDR_ANY.
       */
     BOOL StartListener(
       const H323TransportAddress & iface /// Address of interface to listen on.
     );
 
+    /**Add listeners to the endpoint.
+       Set the collection of listeners which will allow the automatic
+       creating of incoming call connections. An application should use
+       OnConnectionEstablished() to monitor when calls have arrived and been
+       successfully negotiated.
+
+       If a listener already exists on the interface specified in the list
+       then it is ignored. If a listener does not yet exist a new one is
+       created and if a listener is running that is not in the list then it
+       is stopped and removed.
+
+       If the array is empty then the string "*" is assumed which will listen
+       on the standard TCP port on INADDR_ANY.
+
+       Returns TRUE if at least one interface was successfully started.
+      */
+    BOOL StartListeners(
+      const H323TransportAddressArray & ifaces /// Interfaces to listen on.
+    );
+
     /**Remove a listener from the endpoint.
        If the listener parameter is NULL then all listeners are removed.
       */
@@ -378,6 +577,14 @@ class H323EndPoint : public PObject
       H323Listener * listener /// Transport dependent listener.
     );
 
+    /**Return a list of the transport addresses for all listeners on this endpoint
+      */
+    H323TransportAddressArray GetInterfaceAddresses(
+      BOOL excludeLocalHost = TRUE,       /// Flag to exclude 127.0.0.1
+      H323Transport * associatedTransport = NULL
+                          /// Associated transport for precedence and translation
+    );
+
     /**Make a call to a remote party. An appropriate transport is determined
        from the remoteParty parameter. The general form for this parameter is
        [alias@][transport$]host[:port] where the default alias is the same as
@@ -483,7 +690,7 @@ class H323EndPoint : public PObject
     void TransferCall(
       const PString & token,        /// Existing connection to be transferred
       const PString & remoteParty,  /// Remote party to transfer the existing call to
-      const PString & callIdentity = PString()
+      const PString & callIdentity = PString::Empty()
                                     /// Call Identity of secondary call if present
     );
 
@@ -506,17 +713,35 @@ class H323EndPoint : public PObject
       BOOL localHold   /// true for Local Hold, false for Remote Hold
     );
 
+    /** Initiate Call intrusion
+        Designed similar to MakeCall function
+      */
+    H323Connection * IntrudeCall(
+      const PString & remoteParty,  /// Remote party to intrude call
+      PString & token,              /// String to receive token for connection
+      unsigned capabilityLevel,     /// Capability level
+      void * userData = NULL        /// user data to pass to CreateConnection
+    );
+
+    H323Connection * IntrudeCall(
+      const PString & remoteParty,  /// Remote party to intrude call
+      H323Transport * transport,    /// Transport to use for call.
+      PString & token,              /// String to receive token for connection
+      unsigned capabilityLevel,     /// Capability level
+      void * userData = NULL        /// user data to pass to CreateConnection
+    );
+
     /**Parse a party address into alias and transport components.
        An appropriate transport is determined from the remoteParty parameter.
        The general form for this parameter is [alias@][transport$]host[:port]
        where the default alias is the same as the host, the default transport
        is "ip" and the default port is 1720.
       */
-    void ParsePartyName(
+    BOOL ParsePartyName(
       const PString & party,          /// Party name string.
       PString & alias,                /// Parsed alias name
       H323TransportAddress & address  /// Parsed transport address
-    ) const;
+    );
 
     /**Clear a current connection.
        This hangs up the connection to a remote endpoint. Note that this function
@@ -576,6 +801,10 @@ class H323EndPoint : public PObject
       const PString & token     /// Token to identify connection
     );
 
+    /**Get all calls current on the endpoint.
+      */
+    PStringList GetAllConnections();
+
     /**Call back for incoming call.
        This function is called from the OnReceivedSignalSetup() function
        before it sends the Alerting PDU. It gives an opportunity for an
@@ -597,7 +826,7 @@ class H323EndPoint : public PObject
        This gives the application an opportunity to abort the transfer.
        The default behaviour just returns TRUE.
       */
-    BOOL OnCallTransferInitiate(
+    virtual BOOL OnCallTransferInitiate(
       H323Connection & connection,    /// Connection to transfer
       const PString & remoteParty     /// Party transferring to.
     );
@@ -651,11 +880,13 @@ class H323EndPoint : public PObject
       const PString & user                /// Username of remote endpoint
     );
 
-    /**A call back function whenever connection indicates it should be
-       forwarded.
-       Return TRUE if the call is being redirected.
+    /**A call back function when a connection indicates it is to be forwarded.
+       An H323 application may handle this call back so it can make
+       complicated decisions on if the call forward ius to take place. If it
+       decides to do so it must call MakeCall() and return TRUE.
 
-       The default behaviour simply returns FALSE.
+       The default behaviour simply returns FALSE and that the automatic
+       call forwarding should take place. See ForwardConnection()
       */
     virtual BOOL OnConnectionForwarded(
       H323Connection & connection,    /// Connection to be forwarded
@@ -663,6 +894,20 @@ class H323EndPoint : public PObject
       const H323SignalPDU & pdu       /// Full PDU initiating forwarding
     );
 
+    /**Forward the call using the same token as the specified connection.
+       Return TRUE if the call is being redirected.
+
+       The default behaviour will replace the current call in the endpoints
+       call list using the same token as the call being redirected. Not that
+       even though the same token is being used the actual object is
+       completely mad anew.
+      */
+    virtual BOOL ForwardConnection(
+      H323Connection & connection,    /// Connection to be forwarded
+      const PString & forwardParty,   /// Remote party to forward to
+      const H323SignalPDU & pdu       /// Full PDU initiating forwarding
+    );
+
     /**A call back function whenever a connection is established.
        This indicates that a connection to a remote endpoint was established
        with a control channel and zero or more logical channels.
@@ -822,12 +1067,62 @@ class H323EndPoint : public PObject
     );
   //@}
 
+  /**@name Service Control */
+  //@{
+    /**Call back for HTTP based Service Control.
+       An application may override this to use an HTTP based channel using a
+       resource designated by the session ID. For example the session ID can
+       correspond to a browser window and the 
+
+       The default behaviour does nothing.
+      */
+    virtual void OnHTTPServiceControl(
+      unsigned operation,  /// Control operation
+      unsigned sessionId,  /// Session ID for HTTP page
+      const PString & url  /// URL to use.
+    );
+
+    /**Call back for call credit information.
+       An application may override this to display call credit information
+       on registration, or when a call is started.
+
+       The canDisplayAmountString member variable must also be set to TRUE
+       for this to operate.
+
+       The default behaviour does nothing.
+      */
+    virtual void OnCallCreditServiceControl(
+      const PString & amount,  /// UTF-8 string for amount, including currency.
+      BOOL mode          /// Flag indicating that calls will debit the account.
+    );
+
+    /**Handle incoming service control session information.
+       Default behaviour calls session.OnChange()
+     */
+    virtual void OnServiceControlSession(
+      unsigned type,
+      unsigned sessionid,
+      const H323ServiceControlSession & session,
+      H323Connection * connection
+    );
+
+    /**Create the service control session object.
+     */
+    virtual H323ServiceControlSession * CreateServiceControlSession(
+      const H225_ServiceControlDescriptor & contents
+    );
+  //@}
+
   /**@name Other services */
   //@{
     /**Create an instance of the T.120 protocol handler.
        This is called when the OpenLogicalChannel subsystem requires that
        a T.120 channel be established.
 
+       Note that if the application overrides this it should return a pointer to a
+       heap variable (using new) as it will be automatically deleted when the
+       H323Connection is deleted.
+
        The default behavour returns NULL.
       */
     virtual OpalT120Protocol * CreateT120ProtocolHandler(
@@ -838,6 +1133,10 @@ class H323EndPoint : public PObject
        This is called when the OpenLogicalChannel subsystem requires that
        a T.38 fax channel be established.
 
+       Note that if the application overrides this it should return a pointer to a
+       heap variable (using new) as it will be automatically deleted when the
+       H323Connection is deleted.
+
        The default behavour returns NULL.
       */
     virtual OpalT38Protocol * CreateT38ProtocolHandler(
@@ -855,7 +1154,7 @@ class H323EndPoint : public PObject
        Additional aliases may be added by the use of the AddAliasName()
        function, however that list will be cleared when this function is used.
      */
-    void SetLocalUserName(
+    virtual void SetLocalUserName(
       const PString & name  /// Local name of endpoint (prime alias)
     );
 
@@ -863,7 +1162,7 @@ class H323EndPoint : public PObject
        defaults to the logged in user as obtained from the
        PProcess::GetUserName() function.
      */
-    const PString & GetLocalUserName() const { return localAliasNames[0]; }
+    virtual const PString & GetLocalUserName() const { return localAliasNames[0]; }
 
     /**Add an alias name to be used for the local end of any connections. If
        the alias name already exists in the list then is is not added again.
@@ -888,6 +1187,16 @@ class H323EndPoint : public PObject
      */
     const PStringList & GetAliasNames() const { return localAliasNames; }
 
+    /**Get the default ILS server to use for user lookup.
+      */
+    const PString & GetDefaultILSServer() const { return ilsServer; }
+
+    /**Set the default ILS server to use for user lookup.
+      */
+    void SetDefaultILSServer(
+      const PString & server
+      ) { ilsServer = server; }
+
     /**Get the default fast start mode.
       */
     BOOL IsFastStartDisabled() const
@@ -921,6 +1230,49 @@ class H323EndPoint : public PObject
       BOOL mode /// New default mode
     ) { disableH245inSetup = mode; } 
 
+    /**Get the detect in-band DTMF flag.
+      */
+    BOOL DetectInBandDTMFDisabled() const
+      { return disableDetectInBandDTMF; }
+
+    /**Set the detect in-band DTMF flag.
+      */
+    void DisableDetectInBandDTMF(
+      BOOL mode /// New default mode
+    ) { disableDetectInBandDTMF = mode; } 
+
+    /**Get the flag indicating the endpoint can display an amount string.
+      */
+    BOOL CanDisplayAmountString() const
+      { return canDisplayAmountString; }
+
+    /**Set the flag indicating the endpoint can display an amount string.
+      */
+    void SetCanDisplayAmountString(
+      BOOL mode /// New default mode
+    ) { canDisplayAmountString = mode; } 
+
+    /**Get the flag indicating the call will automatically clear after a time.
+      */
+    BOOL CanEnforceDurationLimit() const
+      { return canEnforceDurationLimit; }
+
+    /**Set the flag indicating the call will automatically clear after a time.
+      */
+    void SetCanEnforceDurationLimit(
+      BOOL mode /// New default mode
+    ) { canEnforceDurationLimit = mode; } 
+
+    /**Get Call Intrusion Protection Level of the end point.
+      */
+    unsigned GetCallIntrusionProtectionLevel() const { return callIntrusionProtectionLevel; }
+
+    /**Set Call Intrusion Protection Level of the end point.
+      */
+    void SetCallIntrusionProtectionLevel(
+      unsigned level  // New level from 0 to 3
+    ) { PAssert(level<=3, PInvalidParameter); callIntrusionProtectionLevel = level; }
+
     /**Set the name for the sound channel to be used for output.
        If the name is not suitable for use with the PSoundChannel class then
        the function will return FALSE and not change the device.
@@ -974,11 +1326,11 @@ class H323EndPoint : public PObject
 
     /**Get the default mode for sending User Input Indications.
       */
-    H323Connection::SendUserInputModes GetSendUserInputModes() const { return defaultSendUserInputMode; }
+    H323Connection::SendUserInputModes GetSendUserInputMode() const { return defaultSendUserInputMode; }
 
     /**Set the default mode for sending User Input Indications.
       */
-    void SetSendUserInputModes(H323Connection::SendUserInputModes mode) { defaultSendUserInputMode = mode; }
+    void SetSendUserInputMode(H323Connection::SendUserInputModes mode) { defaultSendUserInputMode = mode; }
 
     /**See if should auto-start receive video channels on connection.
      */
@@ -996,6 +1348,10 @@ class H323EndPoint : public PObject
      */
     BOOL CanAutoStartTransmitFax() const { return autoStartTransmitFax; }
 
+    /**See if should automatically do call forward of connection.
+     */
+    BOOL CanAutoCallForward() const { return autoCallForward; }
+
     /**Get the set of listeners (incoming call transports) for this endpoint.
      */
     const H323ListenerList & GetListeners() const { return listeners; }
@@ -1044,13 +1400,22 @@ class H323EndPoint : public PObject
      */
     BOOL IsMCU() const;
 
-    /**Get the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio jitter delay parameter.
+       Defaults to 50ms
      */
-    WORD GetMaxAudioDelayJitter() const { return maxAudioDelayJitter; }
+    unsigned GetMinAudioJitterDelay() const { return minAudioJitterDelay; }
 
-    /**Set the default maximum audio delay jitter parameter.
+    /**Get the default maximum audio delay jitter parameter.
+       Defaults to 250ms.
      */
-    void SetMaxAudioDelayJitter(unsigned jitter) { maxAudioDelayJitter = (WORD)jitter; }
+    unsigned GetMaxAudioJitterDelay() const { return maxAudioJitterDelay; }
+
+    /**Set the maximum audio delay jitter parameter.
+     */
+    void SetAudioJitterDelay(
+      unsigned minDelay,   // New minimum jitter buffer delay in milliseconds
+      unsigned maxDelay    // New maximum jitter buffer delay in milliseconds
+    );
 
     /**Get the initial bandwidth parameter.
      */
@@ -1060,34 +1425,82 @@ class H323EndPoint : public PObject
      */
     void SetInitialBandwidth(unsigned bandwidth) { initialBandwidth = bandwidth; }
 
+    /**Return the STUN server to use.
+       Returns NULL if address is a local address as per IsLocalAddress().
+       Always returns the STUN server if address is zero.
+       Note, the pointer is NOT to be deleted by the user.
+      */
+    PSTUNClient * GetSTUN(
+      const PIPSocket::Address & address = 0
+    ) const;
+
+    /**Set the STUN server address, is of the form host[:port]
+      */
+    void SetSTUNServer(
+      const PString & server
+    );
+
+    /**Determine if the address is "local", ie does not need STUN
+     */
+    virtual BOOL IsLocalAddress(
+      const PIPSocket::Address & remoteAddress
+    ) const;
+
+    /**Provide TCP address translation hook
+     */
+    virtual void TranslateTCPAddress(
+      PIPSocket::Address & /*localAddr*/,
+      const PIPSocket::Address & /*remoteAddr */
+    ) { }
+
     /**Get the TCP port number base for H.245 channels
      */
-    WORD GetTCPPortBase() const { return tcpPortBase; }
+    WORD GetTCPPortBase() const { return tcpPorts.base; }
 
     /**Get the TCP port number base for H.245 channels.
      */
-    WORD GetTCPPortMax() const { return tcpPortMax; }
+    WORD GetTCPPortMax() const { return tcpPorts.max; }
 
     /**Set the TCP port number base and max for H.245 channels.
      */
     void SetTCPPorts(unsigned tcpBase, unsigned tcpMax);
 
-    /**Provide TCP address translation hook
+    /**Get the next TCP port number for H.245 channels
      */
-    virtual void TranslateTCPAddress(PIPSocket::Address & /*localAddr*/, const PIPSocket::Address & /*remoteAddr */) { }
+    WORD GetNextTCPPort();
+
+    /**Get the UDP port number base for RAS channels
+     */
+    WORD GetUDPPortBase() const { return udpPorts.base; }
+
+    /**Get the UDP port number base for RAS channels.
+     */
+    WORD GetUDPPortMax() const { return udpPorts.max; }
+
+    /**Set the TCP port number base and max for H.245 channels.
+     */
+    void SetUDPPorts(unsigned udpBase, unsigned udpMax);
+
+    /**Get the next UDP port number for RAS channels
+     */
+    WORD GetNextUDPPort();
 
     /**Get the UDP port number base for RTP channels.
      */
-    WORD GetRtpIpPortBase() const { return rtpIpPortBase; }
+    WORD GetRtpIpPortBase() const { return rtpIpPorts.base; }
 
     /**Get the max UDP port number for RTP channels.
      */
-    WORD GetRtpIpPortMax() const { return rtpIpPortMax; }
+    WORD GetRtpIpPortMax() const { return rtpIpPorts.max; }
 
     /**Set the UDP port number base and max for RTP channels.
      */
     void SetRtpIpPorts(unsigned udpBase, unsigned udpMax);
 
+    /**Get the UDP port number pair for RTP channels.
+     */
+    WORD GetRtpIpPortPair();
+
     /**Get the IP Type Of Service byte for RTP channels.
      */
     BYTE GetRtpIpTypeofService() const { return rtpIpTypeofService; }
@@ -1098,11 +1511,19 @@ class H323EndPoint : public PObject
 
     /**Get the default timeout for calling another endpoint.
      */
-    PTimeInterval GetSignallingChannelCallTimeout() const { return signallingChannelCallTimeout; }
+    const PTimeInterval & GetSignallingChannelCallTimeout() const { return signallingChannelCallTimeout; }
+
+    /**Get the default timeout for incoming H.245 connection.
+     */
+    const PTimeInterval & GetControlChannelStartTimeout() const { return controlChannelStartTimeout; }
+
+    /**Get the default timeout for waiting on an end session.
+     */
+    const PTimeInterval & GetEndSessionTimeout() const { return endSessionTimeout; }
 
     /**Get the default timeout for master slave negotiations.
      */
-    PTimeInterval GetMasterSlaveDeterminationTimeout() const { return masterSlaveDeterminationTimeout; }
+    const PTimeInterval & GetMasterSlaveDeterminationTimeout() const { return masterSlaveDeterminationTimeout; }
 
     /**Get the default retries for H245 master slave negotiations.
      */
@@ -1110,47 +1531,39 @@ class H323EndPoint : public PObject
 
     /**Get the default timeout for H245 capability exchange negotiations.
      */
-    PTimeInterval GetCapabilityExchangeTimeout() const { return capabilityExchangeTimeout; }
+    const PTimeInterval & GetCapabilityExchangeTimeout() const { return capabilityExchangeTimeout; }
 
     /**Get the default timeout for H245 logical channel negotiations.
      */
-    PTimeInterval GetLogicalChannelTimeout() const { return logicalChannelTimeout; }
+    const PTimeInterval & GetLogicalChannelTimeout() const { return logicalChannelTimeout; }
 
     /**Get the default timeout for H245 request mode negotiations.
      */
-    PTimeInterval GetRequestModeTimeout() const { return logicalChannelTimeout; }
+    const PTimeInterval & GetRequestModeTimeout() const { return logicalChannelTimeout; }
 
     /**Get the default timeout for H245 round trip delay negotiations.
      */
-    PTimeInterval GetRoundTripDelayTimeout() const { return roundTripDelayTimeout; }
+    const PTimeInterval & GetRoundTripDelayTimeout() const { return roundTripDelayTimeout; }
 
     /**Get the default rate H245 round trip delay is calculated by connection.
      */
-    PTimeInterval GetRoundTripDelayRate() const { return roundTripDelayRate; }
+    const PTimeInterval & GetRoundTripDelayRate() const { return roundTripDelayRate; }
 
-    /**Get the default rate H245 round trip delay is calculated by connection.
+    /**Get the flag for clearing a call if the round trip delay calculation fails.
      */
     BOOL ShouldClearCallOnRoundTripFail() const { return clearCallOnRoundTripFail; }
 
-    /**Get the default timeout for Call Transfer Timer CT-T1.
+    /**Get the amount of time with no media that should cause call to clear
      */
-    PTimeInterval GetCallTransferT1() const { return callTransferT1; }
+    const PTimeInterval & GetNoMediaTimeout() const;
 
-    /**Get the default timeout for Call Transfer Timer CT-T2.
+    /**Set the amount of time with no media that should cause call to clear
      */
-    PTimeInterval GetCallTransferT2() const { return callTransferT2; }
-
-    /**Get the default timeout for Call Transfer Timer CT-T3.
-     */
-    PTimeInterval GetCallTransferT3() const { return callTransferT3; }
-
-    /**Get the default timeout for Call Transfer Timer CT-T4.
-     */
-    PTimeInterval GetCallTransferT4() const { return callTransferT4; }
+    BOOL SetNoMediaTimeout(PTimeInterval newInterval);
 
     /**Get the default timeout for GatekeeperRequest and Gatekeeper discovery.
      */
-    PTimeInterval GetGatekeeperRequestTimeout() const { return gatekeeperRequestTimeout; }
+    const PTimeInterval & GetGatekeeperRequestTimeout() const { return gatekeeperRequestTimeout; }
 
     /**Get the default retries for GatekeeperRequest and Gatekeeper discovery.
      */
@@ -1158,7 +1571,7 @@ class H323EndPoint : public PObject
 
     /**Get the default timeout for RAS protocol transactions.
      */
-    PTimeInterval GetRasRequestTimeout() const { return rasRequestTimeout; }
+    const PTimeInterval & GetRasRequestTimeout() const { return rasRequestTimeout; }
 
     /**Get the default retries for RAS protocol transations.
      */
@@ -1167,11 +1580,39 @@ class H323EndPoint : public PObject
     /**Get the default time for gatekeeper to reregister.
        A value of zero disables the keep alive facility.
      */
-    PTimeInterval GetGatekeeperTimeToLive() const { return registrationTimeToLive; }
+    const PTimeInterval & GetGatekeeperTimeToLive() const { return registrationTimeToLive; }
 
     /**Get the iNow Gatekeeper Access Token OID.
      */
-    const PString GetGkAccessTokenOID() const { return gkAccessTokenOID; }
+    const PString & GetGkAccessTokenOID() const { return gkAccessTokenOID; }
+
+    /**Set the iNow Gatekeeper Access Token OID.
+     */
+    void SetGkAccessTokenOID(const PString & token) { gkAccessTokenOID = token; }
+
+    /**Get the default timeout for Call Transfer Timer CT-T1.
+     */
+    const PTimeInterval & GetCallTransferT1() const { return callTransferT1; }
+
+    /**Get the default timeout for Call Transfer Timer CT-T2.
+     */
+    const PTimeInterval & GetCallTransferT2() const { return callTransferT2; }
+
+    /**Get the default timeout for Call Transfer Timer CT-T3.
+     */
+    const PTimeInterval & GetCallTransferT3() const { return callTransferT3; }
+
+    /**Get the default timeout for Call Transfer Timer CT-T4.
+     */
+    const PTimeInterval & GetCallTransferT4() const { return callTransferT4; }
+
+    /** Get Call Intrusion timers timeout */
+    const PTimeInterval & GetCallIntrusionT1() const { return callIntrusionT1; }
+    const PTimeInterval & GetCallIntrusionT2() const { return callIntrusionT2; }
+    const PTimeInterval & GetCallIntrusionT3() const { return callIntrusionT3; }
+    const PTimeInterval & GetCallIntrusionT4() const { return callIntrusionT4; }
+    const PTimeInterval & GetCallIntrusionT5() const { return callIntrusionT5; }
+    const PTimeInterval & GetCallIntrusionT6() const { return callIntrusionT6; }
 
     /**Get the default stack size of cleaner threads.
      */
@@ -1214,13 +1655,13 @@ class H323EndPoint : public PObject
     H323Connection * InternalMakeCall(
       const PString & existingToken,/// Existing connection to be transferred
       const PString & callIdentity, /// Call identity of the secondary call (if it exists)
+      unsigned capabilityLevel,     /// Intrusion capability level
       const PString & remoteParty,  /// Remote party to call
       H323Transport * transport,    /// Transport to use for call.
-      PString & token,              /// String to receive token for connection
-      void * userData = NULL        /// user data to pass to CreateConnection
+      PString & token,              /// String to use/receive token for connection
+      void * userData               /// user data to pass to CreateConnection
     );
 
-
     // Configuration variables, commonly changed
     PStringList localAliasNames;
     PString     soundChannelPlayDevice;
@@ -1231,27 +1672,25 @@ class H323EndPoint : public PObject
     BOOL        autoStartTransmitVideo;
     BOOL        autoStartReceiveFax;
     BOOL        autoStartTransmitFax;
+    BOOL        autoCallForward;
     BOOL        disableFastStart;
     BOOL        disableH245Tunneling;
     BOOL        disableH245inSetup;
+    BOOL        disableDetectInBandDTMF;
+    BOOL        canDisplayAmountString;
+    BOOL        canEnforceDurationLimit;
+    unsigned    callIntrusionProtectionLevel;
     H323AudioCodec::SilenceDetectionMode defaultSilenceDetection;
     H323Connection::SendUserInputModes defaultSendUserInputMode;
 
-    BYTE          t35CountryCode;
-    BYTE          t35Extension;
-    WORD          manufacturerCode;
-    TerminalTypes terminalType;
-
-    WORD     maxAudioDelayJitter;
-    unsigned initialBandwidth;  // in 100s of bits/sev
-    BOOL     clearCallOnRoundTripFail;
+    PString     ilsServer;
 
     // Some more configuration variables, rarely changed.
     unsigned      soundChannelBuffers;
-    WORD          rtpIpPortBase, rtpIpPortMax;
-    WORD          tcpPortBase, tcpPortMax;
     BYTE          rtpIpTypeofService;
     PTimeInterval signallingChannelCallTimeout;
+    PTimeInterval controlChannelStartTimeout;
+    PTimeInterval endSessionTimeout;
     PTimeInterval masterSlaveDeterminationTimeout;
     unsigned      masterSlaveDeterminationRetries;
     PTimeInterval capabilityExchangeTimeout;
@@ -1259,6 +1698,7 @@ class H323EndPoint : public PObject
     PTimeInterval requestModeTimeout;
     PTimeInterval roundTripDelayTimeout;
     PTimeInterval roundTripDelayRate;
+    PTimeInterval noMediaTimeout;
     PTimeInterval gatekeeperRequestTimeout;
     unsigned      gatekeeperRequestRetries;
     PTimeInterval rasRequestTimeout;
@@ -1266,6 +1706,34 @@ class H323EndPoint : public PObject
     PTimeInterval registrationTimeToLive;
     PString       gkAccessTokenOID;
 
+    unsigned minAudioJitterDelay;
+    unsigned maxAudioJitterDelay;
+    unsigned initialBandwidth;  // in 100s of bits/sev
+    BOOL     clearCallOnRoundTripFail;
+
+    struct PortInfo {
+      void Set(
+        unsigned base,
+        unsigned max,
+        unsigned range,
+        unsigned dflt
+      );
+      WORD GetNext(
+        unsigned increment
+      );
+
+      PMutex mutex;
+      WORD   base;
+      WORD   max;
+      WORD   current;
+    } tcpPorts, udpPorts, rtpIpPorts;
+    PSTUNClient * stun;
+
+    BYTE          t35CountryCode;
+    BYTE          t35Extension;
+    WORD          manufacturerCode;
+    TerminalTypes terminalType;
+
     /* Protect against absence of a response to the ctIdentify reqest
        (Transferring Endpoint - Call Transfer with a secondary Call) */
     PTimeInterval callTransferT1;
@@ -1279,6 +1747,14 @@ class H323EndPoint : public PObject
        ctSetup request (Transferred Endpoint) */
     PTimeInterval callTransferT4;
 
+    /** Call Intrusion Timers */
+    PTimeInterval callIntrusionT1;
+    PTimeInterval callIntrusionT2;
+    PTimeInterval callIntrusionT3;
+    PTimeInterval callIntrusionT4;
+    PTimeInterval callIntrusionT5;
+    PTimeInterval callIntrusionT6;
+
     PINDEX cleanerThreadStackSize;
     PINDEX listenerThreadStackSize;
     PINDEX signallingThreadStackSize;
@@ -1296,13 +1772,14 @@ class H323EndPoint : public PObject
     H323ConnectionDict       connectionsActive;
     H323CallIdentityDict     secondaryConenctionsActive;
     PMutex                   connectionsMutex;
+    PMutex                   noMediaMutex;
     PStringSet               connectionsToBeCleaned;
     H323ConnectionsCleaner * connectionsCleaner;
     PSyncPoint               connectionsAreCleaned;
 };
 
 
-#endif // __H323_H
+#endif // __OPAL_H323EP_H
 
 
 /////////////////////////////////////////////////////////////////////////////
