@@ -24,6 +24,16 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: vblasterlid.h,v $
+ * Revision 1.5  2002/09/16 01:14:15  robertj
+ * Added #define so can select if #pragma interface/implementation is used on
+ *   platform basis (eg MacOS) rather than compiler, thanks Robert Monaghan.
+ *
+ * Revision 1.4  2002/09/03 06:19:37  robertj
+ * Normalised the multi-include header prevention ifdef/define symbol.
+ *
+ * Revision 1.3  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
  * Revision 1.2  2002/01/15 07:23:24  craigs
  * Added IsDevicePresent command
  *
@@ -33,8 +43,13 @@
  *
  */
 
-#ifndef __VBLASTER_LID_H
-#define __VBLASTER_LID_H
+#ifndef __OPAL_VBLASTERLID_H
+#define __OPAL_VBLASTERLID_H
+
+#ifdef P_USE_PRAGMA
+#pragma interface
+#endif
+
 
 #define HAS_VBLASTER
 
@@ -43,9 +58,7 @@
 
 #include <ptclib/delaychan.h>
 
-#ifdef __GNUC__
-#pragma interface
-#endif
+
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -608,6 +621,7 @@ class OpalVoipBlasterDevice : public OpalLineInterfaceDevice
 };
 
 
-#endif 
+#endif // __OPAL_VBLASTERLID_H
+
 
 /////////////////////////////////////////////////////////////////////////////
