@@ -27,6 +27,23 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: codecs.cxx,v $
+ * Revision 1.81  2003/07/16 10:43:13  csoutheren
+ * Added SwapChannel function to H323Codec to allow media hold channels
+ * to work better. Thanks to Federico Pinna
+ *
+ * Revision 1.80  2002/12/16 09:11:19  robertj
+ * Added new video bit rate control, thanks Walter H. Whitlock
+ *
+ * Revision 1.79  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.78  2002/05/29 04:48:48  robertj
+ * Changed framed codec so if cannot decode frame just plays silence instead
+ *   of returning error and thus shutting down channel, thanks Federico Pinna
+ *
+ * Revision 1.77  2002/04/16 03:27:54  dereks
+ * Correct logical flaw in CloseRawDataChannel method.
+ *
  * Revision 1.76  2002/04/05 00:52:17  dereks
  * Minor tweaks to cope with received h261 messages.
  *
@@ -273,12 +290,14 @@
  *
  */
 
+#include <ptlib.h>
+
 #ifdef __GNUC__
 #pragma implementation "codecs.h"
 #endif
 
-#include <ptlib.h>
 #include "codecs.h"
+
 #include "channels.h"
 #include "h323pdu.h"
 #include "h323con.h"
@@ -355,22 +374,35 @@ BOOL H323Codec::AttachChannel(PChannel * channel, BOOL autoDelete)
   if (channel == NULL)
     return FALSE;
 
-  if (channel->IsOpen())
-    return TRUE;
+  return channel->IsOpen();
+}
 
-  return FALSE;
+PChannel * H323Codec::SwapChannel(PChannel * newChannel, BOOL autoDelete)
+{
+  PWaitAndSignal mutex(rawChannelMutex);
+
+  PChannel * oldChannel = rawDataChannel;
+
+  rawDataChannel = newChannel;
+  deleteChannel = autoDelete;
+
+  return oldChannel;
 }
 
 
 BOOL H323Codec::CloseRawDataChannel()
 {
-  if ( (rawDataChannel != NULL) && deleteChannel) {   
-     rawDataChannel->Close();
+  if (rawDataChannel == NULL)
+    return FALSE;
+  
+  BOOL closeOK = rawDataChannel->Close();
+  
+  if (deleteChannel) {
      delete rawDataChannel;
      rawDataChannel = NULL;
   }
   
-  return TRUE;
+  return closeOK;
 }  
 
 
@@ -449,9 +481,12 @@ H323VideoCodec::H323VideoCodec(const char * fmt, Direction dir)
   : H323Codec(fmt, dir)
 {
   frameWidth = frameHeight = 0;
-  bytesSent = 0;
-  videoBitRateControlModes = DynamicVideoQuality;
-  startTime = PTimer::Tick();
+  targetFrameTimeMs = 0;
+  videoBitRateControlModes = None;
+
+  oldLength = 0;
+  oldTime = 0;
+  newTime = 0;
 }
 
 
@@ -599,29 +634,27 @@ void H323VideoCodec::Close()
 }
 
 
-BOOL H323VideoCodec::SetAverageBitRate(unsigned bitRate, unsigned modes)
+BOOL H323VideoCodec::SetMaxBitRate(unsigned bitRate)
 {
-  PTRACE(1,"SetAverageBandwidth for video to " << bitRate << "bps");
+  PTRACE(1,"Set bitRateHighLimit for video to " << bitRate << " bps");
         
-  if (bitRate == 0) {       // disable constant bitrate
-    frameRateDivider = 1;
-    videoBitRate = bitRate;
-    return TRUE;    
-  }
+  bitRateHighLimit = bitRate;
 
-  if (bitRate < 1024)
-    bitRate = 1024;      // from 1kbit/s
-  else if (bitRate > 2048*1024)
-    bitRate = 2048*1024;  // to 2Mbit/s
-
-  videoBitRate    = bitRate;
-  defBitsPerFrame = (int)((float)videoBitRate / estFps);
-  reqBitsPerFrame = defBitsPerFrame*frameRateDivider;
-
-  videoBitRateControlModes = modes;
+  if (0 == bitRateHighLimit) // disable bitrate control
+    videoBitRateControlModes &= ~AdaptivePacketDelay;
   return TRUE;
 }
 
+BOOL H323VideoCodec::SetTargetFrameTimeMs(unsigned ms)
+{
+  PTRACE(1,"Set targetFrameTimeMs for video to " << ms << " milliseconds");
+
+  targetFrameTimeMs = ms;
+
+  if (0 == targetFrameTimeMs)
+    videoBitRateControlModes &= ~DynamicVideoQuality;
+  return TRUE;
+ }
 
 void H323VideoCodec::SendMiscCommand(unsigned command)
 {
@@ -901,22 +934,25 @@ BOOL H323FramedAudioCodec::Write(const BYTE * buffer,
 
   // If length is zero then it indicates silence, do nothing.
   PINDEX bytesOfPCM = samplesPerFrame*2;
-  if (length == 0) {
-    memset(sampleBuffer.GetPointer(samplesPerFrame), 0, bytesOfPCM);
-    written = 0;
-  }
-  else {
+  written = 0;
+
+  if (length != 0) {
     if (length > bytesPerFrame)
       length = bytesPerFrame;
     written = bytesPerFrame;
 
     // Decode the data
-    if (!DecodeFrame(buffer, length, written))
-      return FALSE;
-
-    bytesOfPCM = bytesOfPCM*written/bytesPerFrame;
+    if (DecodeFrame(buffer, length, written))
+      bytesOfPCM = bytesOfPCM*written/bytesPerFrame;
+    else {
+      written = length;
+      length = 0;
+    }
   }
 
+  if (length == 0)
+    memset(sampleBuffer.GetPointer(samplesPerFrame), 0, bytesOfPCM);
+
   // Write as 16bit PCM to sound channel
   return WriteRaw(sampleBuffer.GetPointer(), bytesOfPCM);
 }
