@@ -27,6 +27,59 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323caps.cxx,v $
+ * Revision 1.66  2003/06/06 02:13:48  rjongbloed
+ * Changed non-standard capability semantics so can use C style strings as
+ *   the embedded data block (ie automatically call strlen)
+ *
+ * Revision 1.65  2003/05/16 07:30:20  rjongbloed
+ * Fixed correct matching of OLC data types to capabilities, for example CIF
+ *   and QCIF video are different and should match exactly.
+ *
+ * Revision 1.64  2003/04/28 07:00:09  robertj
+ * Fixed problem with compiler(s) not correctly initialising static globals
+ *
+ * Revision 1.63  2003/04/27 23:50:38  craigs
+ * Made list of registered codecs available outside h323caps.cxx
+ *
+ * Revision 1.62  2003/03/18 05:11:22  robertj
+ * Fixed OID based non-standard capabilities, thanks Philippe Massicotte
+ *
+ * Revision 1.61  2002/12/05 12:29:31  rogerh
+ * Add non standard codec identifier for Xiph.org
+ *
+ * Revision 1.60  2002/11/27 11:47:09  robertj
+ * Fixed GNU warning
+ *
+ * Revision 1.59  2002/11/26 22:48:18  craigs
+ * Changed nonStd codec identification to use a table for MS codecs
+ *
+ * Revision 1.58  2002/11/26 13:52:59  craigs
+ * Added PrintOn function for outputting names of nonStandard codecs
+ *
+ * Revision 1.57  2002/11/09 04:44:24  robertj
+ * Fixed function to add capabilities to return correct table entry index.
+ * Cosmetic changes.
+ *
+ * Revision 1.56  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.55  2002/07/18 08:28:52  robertj
+ * Adjusted some trace log levels
+ *
+ * Revision 1.54  2002/06/04 07:16:14  robertj
+ * Fixed user indications (DTMF) not working on some endpoints which indicated
+ *   receiveAndTransmitUserInputCapability in TCS.
+ *
+ * Revision 1.53  2002/05/29 03:55:21  robertj
+ * Added protocol version number checking infrastructure, primarily to improve
+ *   interoperability with stacks that are unforgiving of new features.
+ *
+ * Revision 1.52  2002/05/14 23:20:03  robertj
+ * Fixed incorrect comparison in non-standard capability, tnanks Vyacheslav Frolov
+ *
+ * Revision 1.51  2002/05/10 05:45:41  robertj
+ * Added the max bit rate field to the data channel capability class.
+ *
  * Revision 1.50  2002/03/26 05:51:12  robertj
  * Forced RFC2833 to payload type 101 as some IOS's go nuts otherwise.
  *
@@ -201,13 +254,14 @@
  *
  */
 
+#include <ptlib.h>
+
 #ifdef __GNUC__
 #pragma implementation "h323caps.h"
 #endif
 
-
-#include <ptlib.h>
 #include "h323caps.h"
+
 #include "h323ep.h"
 #include "h225.h"
 #include "h245.h"
@@ -224,12 +278,31 @@ H323_REGISTER_CAPABILITY_FUNCTION(H323_G711uLaw64Capability, "G.711-uLaw-64k{sw}
 }
 
 
-static H323CapabilityRegistration * RegisteredCapabilitiesListHead;
+H323CapabilityRegistration * H323CapabilityRegistration::registeredCapabilitiesListHead = NULL;
 
 
 #define new PNEW
 
 
+#if PTRACING
+ostream & operator<<(ostream & o , H323Capability::MainTypes t)
+{
+  const char * const names[] = {
+    "Audio", "Video", "Data", "UserInput"
+  };
+  return o << names[t];
+}
+
+ostream & operator<<(ostream & o , H323Capability::CapabilityDirection d)
+{
+  const char * const names[] = {
+    "Unknown", "Receive", "Transmit", "ReceiveAndTransmit", "NoDirection"
+  };
+  return o << names[d];
+}
+#endif
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 H323Capability::H323Capability()
@@ -273,7 +346,8 @@ void H323Capability::PrintOn(ostream & strm) const
 
 H323Capability * H323Capability::Create(H323EndPoint & ep, const PString & name)
 {
-  H323CapabilityRegistration * find = RegisteredCapabilitiesListHead;
+  PWaitAndSignal mutex(H323CapabilityRegistration::GetMutex());
+  H323CapabilityRegistration * find = H323CapabilityRegistration::registeredCapabilitiesListHead;
   while (find != NULL) {
     if (*find == name)
       return find->Create(ep);
@@ -349,24 +423,11 @@ BOOL H323Capability::OnReceivedPDU(const H245_Capability & cap)
 }
 
 
-#if PTRACING
-ostream & operator<<(ostream & o , H323Capability::MainTypes t)
+BOOL H323Capability::IsUsable(const H323Connection &) const
 {
-  const char * const names[] = {
-    "Audio", "Video", "Data", "UserInput"
-  };
-  return o << names[t];
+  return TRUE;
 }
 
-ostream & operator<<(ostream & o , H323Capability::CapabilityDirection d)
-{
-  const char * const names[] = {
-    "Unknown", "Receive", "Transmit", "ReceiveAndTransmit", "NoDirection"
-  };
-  return o << names[d];
-}
-#endif
-
 
 /////////////////////////////////////////////////////////////////////////////
 
@@ -386,7 +447,8 @@ H323NonStandardCapabilityInfo::H323NonStandardCapabilityInfo(H323EndPoint & endp
                                                              PINDEX dataSize,
                                                              PINDEX _offset,
                                                              PINDEX _len)
-  : nonStandardData(dataPtr, dataSize),
+  : nonStandardData(dataPtr, dataSize == 0 && dataPtr != NULL
+                                 ? strlen((const char *)dataPtr) : dataSize),
     comparisonOffset(_offset),
     comparisonLength(_len)
 {
@@ -404,7 +466,8 @@ H323NonStandardCapabilityInfo::H323NonStandardCapabilityInfo(const PString & _oi
                                                              PINDEX _offset,
                                                              PINDEX _len)
   : oid(_oid),
-    nonStandardData(dataPtr, dataSize),
+    nonStandardData(dataPtr, dataSize == 0 && dataPtr != NULL
+                                 ? strlen((const char *)dataPtr) : dataSize),
     comparisonOffset(_offset),
     comparisonLength(_len)
 {
@@ -421,7 +484,8 @@ H323NonStandardCapabilityInfo::H323NonStandardCapabilityInfo(BYTE country,
   : t35CountryCode(country),
     t35Extension(extension),
     manufacturerCode(maufacturer),
-    nonStandardData(dataPtr, dataSize),
+    nonStandardData(dataPtr, dataSize == 0 && dataPtr != NULL
+                                 ? strlen((const char *)dataPtr) : dataSize),
     comparisonOffset(_offset),
     comparisonLength(_len)
 {
@@ -462,7 +526,8 @@ BOOL H323NonStandardCapabilityInfo::OnSendingNonStandardPDU(PASN_Choice & pdu,
 
   if (!oid) {
     param.m_nonStandardIdentifier.SetTag(H245_NonStandardIdentifier::e_object);
-    ((PASN_ObjectId &)param) = oid;
+    PASN_ObjectId & nonStandardIdentifier = param.m_nonStandardIdentifier;
+    nonStandardIdentifier = oid;
   }
   else {
     param.m_nonStandardIdentifier.SetTag(H245_NonStandardIdentifier::e_h221NonStandard);
@@ -498,7 +563,8 @@ PObject::Comparison H323NonStandardCapabilityInfo::CompareParam(const H245_NonSt
     if (param.m_nonStandardIdentifier.GetTag() != H245_NonStandardIdentifier::e_object)
       return PObject::LessThan;
 
-    return oid.Compare((PASN_ObjectId &)param);
+    const PASN_ObjectId & nonStandardIdentifier = param.m_nonStandardIdentifier;
+    return oid.Compare(nonStandardIdentifier.AsString());
   }
 
   if (param.m_nonStandardIdentifier.GetTag() != H245_NonStandardIdentifier::e_h221NonStandard)
@@ -518,7 +584,7 @@ PObject::Comparison H323NonStandardCapabilityInfo::CompareParam(const H245_NonSt
 
   if (h221.m_manufacturerCode < (unsigned)manufacturerCode)
     return PObject::LessThan;
-  if (h221.m_manufacturerCode < (unsigned)manufacturerCode)
+  if (h221.m_manufacturerCode > (unsigned)manufacturerCode)
     return PObject::GreaterThan;
 
 
@@ -674,7 +740,7 @@ BOOL H323AudioCapability::OnReceivedPDU(const H245_Capability & cap)
   H323Capability::OnReceivedPDU(cap);
 
   if (cap.GetTag() != H245_Capability::e_receiveAudioCapability &&
-    cap.GetTag() != H245_Capability::e_receiveAndTransmitAudioCapability)
+      cap.GetTag() != H245_Capability::e_receiveAndTransmitAudioCapability)
     return FALSE;
 
   unsigned packetSize = txFramesInPacket;
@@ -950,6 +1016,12 @@ BOOL H323NonStandardVideoCapability::IsNonStandardMatch(const H245_NonStandardPa
 
 /////////////////////////////////////////////////////////////////////////////
 
+H323DataCapability::H323DataCapability(unsigned rate)
+  : maxBitRate(rate)
+{
+}
+
+
 H323Capability::MainTypes H323DataCapability::GetMainType() const
 {
   return e_Data;
@@ -971,21 +1043,27 @@ H323Codec * H323DataCapability::CreateCodec(H323Codec::Direction) const
 BOOL H323DataCapability::OnSendingPDU(H245_Capability & cap) const
 {
   cap.SetTag(H245_Capability::e_receiveAndTransmitDataApplicationCapability);
-  return OnSendingPDU((H245_DataApplicationCapability &)cap);
+  H245_DataApplicationCapability & app = cap;
+  app.m_maxBitRate = maxBitRate;
+  return OnSendingPDU(app);
 }
 
 
 BOOL H323DataCapability::OnSendingPDU(H245_DataType & dataType) const
 {
   dataType.SetTag(H245_DataType::e_data);
-  return OnSendingPDU((H245_DataApplicationCapability &)dataType);
+  H245_DataApplicationCapability & app = dataType;
+  app.m_maxBitRate = maxBitRate;
+  return OnSendingPDU(app);
 }
 
 
 BOOL H323DataCapability::OnSendingPDU(H245_ModeElement & mode) const
 {
   mode.m_type.SetTag(H245_ModeElement_type::e_dataMode);
-  return OnSendingPDU((H245_DataMode &)mode.m_type);
+  H245_DataMode & type = mode.m_type;
+  type.m_bitRate = maxBitRate;
+  return OnSendingPDU(type);
 }
 
 
@@ -997,7 +1075,9 @@ BOOL H323DataCapability::OnReceivedPDU(const H245_Capability & cap)
       cap.GetTag() != H245_Capability::e_receiveAndTransmitDataApplicationCapability)
     return FALSE;
 
-  return OnReceivedPDU((const H245_DataApplicationCapability &)cap);
+  const H245_DataApplicationCapability & app = cap;
+  maxBitRate = app.m_maxBitRate;
+  return OnReceivedPDU(app);
 }
 
 
@@ -1006,40 +1086,48 @@ BOOL H323DataCapability::OnReceivedPDU(const H245_DataType & dataType, BOOL)
   if (dataType.GetTag() != H245_DataType::e_data)
     return FALSE;
 
-  return OnReceivedPDU((const H245_DataApplicationCapability &)dataType);
+  const H245_DataApplicationCapability & app = dataType;
+  maxBitRate = app.m_maxBitRate;
+  return OnReceivedPDU(app);
 }
 
 
 /////////////////////////////////////////////////////////////////////////////
 
-H323NonStandardDataCapability::H323NonStandardDataCapability(H323EndPoint & endpoint,
+H323NonStandardDataCapability::H323NonStandardDataCapability(unsigned maxBitRate,
+                                                             H323EndPoint & endpoint,
                                                              const BYTE * fixedData,
                                                              PINDEX dataSize,
                                                              PINDEX offset,
                                                              PINDEX length)
-  : H323NonStandardCapabilityInfo(endpoint, fixedData, dataSize, offset, length)
+  : H323DataCapability(maxBitRate),
+    H323NonStandardCapabilityInfo(endpoint, fixedData, dataSize, offset, length)
 {
 }
 
 
-H323NonStandardDataCapability::H323NonStandardDataCapability(const PString & oid,
+H323NonStandardDataCapability::H323NonStandardDataCapability(unsigned maxBitRate,
+                                                             const PString & oid,
                                                              const BYTE * fixedData,
                                                              PINDEX dataSize,
                                                              PINDEX offset,
                                                              PINDEX length)
-  : H323NonStandardCapabilityInfo(oid, fixedData, dataSize, offset, length)
+  : H323DataCapability(maxBitRate),
+    H323NonStandardCapabilityInfo(oid, fixedData, dataSize, offset, length)
 {
 }
 
 
-H323NonStandardDataCapability::H323NonStandardDataCapability(BYTE country,
+H323NonStandardDataCapability::H323NonStandardDataCapability(unsigned maxBitRate,
+                                                             BYTE country,
                                                              BYTE extension,
                                                              WORD maufacturer,
                                                              const BYTE * fixedData,
                                                              PINDEX dataSize,
                                                              PINDEX offset,
                                                              PINDEX length)
-  : H323NonStandardCapabilityInfo(country, extension, maufacturer, fixedData, dataSize, offset, length)
+  : H323DataCapability(maxBitRate),
+    H323NonStandardCapabilityInfo(country, extension, maufacturer, fixedData, dataSize, offset, length)
 {
 }
 
@@ -1099,6 +1187,16 @@ PObject * H323_G711Capability::Clone() const
 }
 
 
+unsigned H323_G711Capability::GetSubType() const
+{
+  static const unsigned G711SubType[2][2] = {
+    { H245_AudioCapability::e_g711Alaw64k, H245_AudioCapability::e_g711Alaw56k },
+    { H245_AudioCapability::e_g711Ulaw64k, H245_AudioCapability::e_g711Ulaw56k }
+  };
+  return G711SubType[mode][speed];
+}
+
+
 PString H323_G711Capability::GetFormatName() const
 {
   static const char * const G711Name[2][2] = {
@@ -1109,18 +1207,6 @@ PString H323_G711Capability::GetFormatName() const
 }
 
 
-static const unsigned G711SubType[2][2] = {
-  { H245_AudioCapability::e_g711Alaw64k, H245_AudioCapability::e_g711Alaw56k },
-  { H245_AudioCapability::e_g711Ulaw64k, H245_AudioCapability::e_g711Ulaw56k }
-};
-
-
-unsigned H323_G711Capability::GetSubType() const
-{
-  return G711SubType[mode][speed];
-}
-
-
 H323Codec * H323_G711Capability::CreateCodec(H323Codec::Direction direction) const
 {
   unsigned packetSize = 8*(direction == H323Codec::Encoder ? txFramesInPacket : rxFramesInPacket);
@@ -1168,12 +1254,6 @@ PObject * H323_UserInputCapability::Clone() const
 }
 
 
-PString H323_UserInputCapability::GetFormatName() const
-{
-  return SubTypeNames[subType];
-}
-
-
 H323Capability::MainTypes H323_UserInputCapability::GetMainType() const
 {
   return e_UserInput;
@@ -1197,6 +1277,12 @@ unsigned  H323_UserInputCapability::GetSubType()  const
 }
 
 
+PString H323_UserInputCapability::GetFormatName() const
+{
+  return SubTypeNames[subType];
+}
+
+
 H323Channel * H323_UserInputCapability::CreateChannel(H323Connection &,
                                                       H323Channel::Directions,
                                                       unsigned,
@@ -1257,7 +1343,8 @@ BOOL H323_UserInputCapability::OnReceivedPDU(const H245_Capability & pdu)
     return TRUE;
   }
 
-  if (pdu.GetTag() != H245_Capability::e_receiveUserInputCapability)
+  if (pdu.GetTag() != H245_Capability::e_receiveUserInputCapability &&
+      pdu.GetTag() != H245_Capability::e_receiveAndTransmitUserInputCapability)
     return FALSE;
 
   const H245_UserInputCapability & ui = pdu;
@@ -1272,6 +1359,18 @@ BOOL H323_UserInputCapability::OnReceivedPDU(const H245_DataType &, BOOL)
 }
 
 
+BOOL H323_UserInputCapability::IsUsable(const H323Connection & connection) const
+{
+  if (connection.GetControlVersion() >= 7)
+    return TRUE;
+
+  if (connection.GetRemoteApplication().Find("AltiServ-ITG") != P_MAX_INDEX)
+    return FALSE;
+
+  return subType != SignalToneRFC2833;
+}
+
+
 void H323_UserInputCapability::AddAllCapabilities(H323Capabilities & capabilities,
                                                   PINDEX descriptorNum,
                                                   PINDEX simultaneous)
@@ -1479,11 +1578,12 @@ PINDEX H323Capabilities::AddAllCapabilities(H323EndPoint & ep,
                                             PINDEX simultaneous,
                                             const PString & name)
 {
-  PINDEX reply = P_MAX_INDEX;
+  PINDEX reply = descriptorNum == P_MAX_INDEX ? P_MAX_INDEX : simultaneous;
 
   PStringArray wildcard = name.Tokenise('*', FALSE);
 
-  H323CapabilityRegistration * reg = RegisteredCapabilitiesListHead;
+  PWaitAndSignal mutex(H323CapabilityRegistration::GetMutex());
+  H323CapabilityRegistration * reg = H323CapabilityRegistration::registeredCapabilitiesListHead;
   while (reg != NULL) {
     if (MatchWildcard(*reg, wildcard) && FindCapability(*reg) == NULL) {
       PINDEX num = SetCapability(descriptorNum, simultaneous, reg->Create(ep));
@@ -1493,7 +1593,7 @@ PINDEX H323Capabilities::AddAllCapabilities(H323EndPoint & ep,
         simultaneous = P_MAX_INDEX;
       }
       else if (simultaneous == P_MAX_INDEX) {
-        if (reply != P_MAX_INDEX)
+        if (reply == P_MAX_INDEX)
           reply = num;
         simultaneous = num;
       }
@@ -1537,7 +1637,7 @@ void H323Capabilities::Add(H323Capability * capability)
   capability->SetCapabilityNumber(MergeCapabilityNumber(table, 1));
   table.Append(capability);
 
-  PTRACE(2, "H323\tAdded capability: " << *capability);
+  PTRACE(3, "H323\tAdded capability: " << *capability);
 }
 
 
@@ -1547,7 +1647,7 @@ H323Capability * H323Capabilities::Copy(const H323Capability & capability)
   newCapability->SetCapabilityNumber(MergeCapabilityNumber(table, capability.GetCapabilityNumber()));
   table.Append(newCapability);
 
-  PTRACE(2, "H323\tAdded capability: " << *newCapability);
+  PTRACE(3, "H323\tAdded capability: " << *newCapability);
   return newCapability;
 }
 
@@ -1557,6 +1657,8 @@ void H323Capabilities::Remove(H323Capability * capability)
   if (capability == NULL)
     return;
 
+  PTRACE(3, "H323\tRemoving capability: " << *capability);
+
   unsigned capabilityNumber = capability->GetCapabilityNumber();
 
   for (PINDEX outer = 0; outer < set.GetSize(); outer++) {
@@ -1604,11 +1706,11 @@ void H323Capabilities::RemoveAll()
 
 H323Capability * H323Capabilities::FindCapability(unsigned capabilityNumber) const
 {
-  PTRACE(3, "H323\tFindCapability: " << capabilityNumber);
+  PTRACE(4, "H323\tFindCapability: " << capabilityNumber);
 
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     if (table[i].GetCapabilityNumber() == capabilityNumber) {
-      PTRACE(2, "H323\tFound capability: " << table[i]);
+      PTRACE(3, "H323\tFound capability: " << table[i]);
       return &table[i];
     }
   }
@@ -1620,7 +1722,7 @@ H323Capability * H323Capabilities::FindCapability(unsigned capabilityNumber) con
 H323Capability * H323Capabilities::FindCapability(const PString & formatName,
                               H323Capability::CapabilityDirection direction) const
 {
-  PTRACE(3, "H323\tFindCapability: \"" << formatName << '"');
+  PTRACE(4, "H323\tFindCapability: \"" << formatName << '"');
 
   PStringArray wildcard = formatName.Tokenise('*', FALSE);
 
@@ -1629,7 +1731,7 @@ H323Capability * H323Capabilities::FindCapability(const PString & formatName,
     if (MatchWildcard(str, wildcard) &&
           (direction == H323Capability::e_Unknown ||
            table[i].GetCapabilityDirection() == direction)) {
-      PTRACE(2, "H323\tFound capability: " << table[i]);
+      PTRACE(3, "H323\tFound capability: " << table[i]);
       return &table[i];
     }
   }
@@ -1641,11 +1743,11 @@ H323Capability * H323Capabilities::FindCapability(const PString & formatName,
 H323Capability * H323Capabilities::FindCapability(
                               H323Capability::CapabilityDirection direction) const
 {
-  PTRACE(1, "H323\tFindCapability: \"" << direction << '"');
+  PTRACE(4, "H323\tFindCapability: \"" << direction << '"');
 
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     if (table[i].GetCapabilityDirection() == direction) {
-      PTRACE(1, "H323\tFound capability: " << table[i]);
+      PTRACE(3, "H323\tFound capability: " << table[i]);
       return &table[i];
     }
   }
@@ -1656,11 +1758,11 @@ H323Capability * H323Capabilities::FindCapability(
 
 H323Capability * H323Capabilities::FindCapability(const H323Capability & capability) const
 {
-  PTRACE(3, "H323\tFindCapability: " << capability);
+  PTRACE(4, "H323\tFindCapability: " << capability);
 
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     if (table[i] == capability) {
-      PTRACE(2, "H323\tFound capability: " << table[i]);
+      PTRACE(3, "H323\tFound capability: " << table[i]);
       return &table[i];
     }
   }
@@ -1721,27 +1823,56 @@ H323Capability * H323Capabilities::FindCapability(const H245_DataType & dataType
 {
   PTRACE(4, "H323\tFindCapability: " << dataType.GetTagName());
 
-  switch (dataType.GetTag()) {
-    case H245_DataType::e_audioData :
+  for (PINDEX i = 0; i < table.GetSize(); i++) {
+    H323Capability & capability = table[i];
+    BOOL checkExact;
+    switch (dataType.GetTag()) {
+      case H245_DataType::e_audioData :
       {
         const H245_AudioCapability & audio = dataType;
-        return FindCapability(H323Capability::e_Audio, audio, H245_AudioCapability::e_nonStandard);
+        checkExact = capability.GetMainType() == H323Capability::e_Audio &&
+                    (capability.GetSubType() == audio.GetTag() ||
+                     capability.GetSubType() == H245_AudioCapability::e_nonStandard &&
+                     audio.GetTag() == H245_AudioCapability::e_nonStandard &&
+                     capability.IsNonStandardMatch((const H245_NonStandardParameter &)audio));
+        break;
       }
 
-    case H245_DataType::e_videoData :
+      case H245_DataType::e_videoData :
       {
         const H245_VideoCapability & video = dataType;
-        return FindCapability(H323Capability::e_Video, video, H245_VideoCapability::e_nonStandard);
+        checkExact = capability.GetMainType() == H323Capability::e_Video &&
+                    (capability.GetSubType() == video.GetTag() ||
+                     capability.GetSubType() == H245_VideoCapability::e_nonStandard &&
+                     video.GetTag() == H245_VideoCapability::e_nonStandard &&
+                     capability.IsNonStandardMatch((const H245_NonStandardParameter &)video));
+        break;
       }
 
-    case H245_DataType::e_data :
+      case H245_DataType::e_data :
       {
         const H245_DataApplicationCapability & data = dataType;
-        return FindCapability(H323Capability::e_Data, data.m_application, H245_DataApplicationCapability_application::e_nonStandard);
+        checkExact = capability.GetMainType() == H323Capability::e_Data &&
+                    (capability.GetSubType() == data.m_application.GetTag() ||
+                     capability.GetSubType() == H245_DataApplicationCapability_application::e_nonStandard &&
+                     data.m_application.GetTag() == H245_DataApplicationCapability_application::e_nonStandard &&
+                     capability.IsNonStandardMatch((const H245_NonStandardParameter &)data.m_application));
+        break;
       }
 
-    default :
-      break;
+      default :
+        checkExact = FALSE;
+    }
+
+    if (checkExact) {
+      H323Capability * compare = (H323Capability *)capability.Clone();
+      if (compare->OnReceivedPDU(dataType, FALSE) && *compare == capability) {
+        delete compare;
+        PTRACE(3, "H323\tFound capability: " << capability);
+        return &capability;
+      }
+      delete compare;
+    }
   }
 
   return NULL;
@@ -1786,14 +1917,14 @@ H323Capability * H323Capabilities::FindCapability(H323Capability::MainTypes main
   if (subTypePDU.GetTag() != nonStandardTag)
     return FindCapability(mainType, subTypePDU.GetTag());
 
-  PTRACE(3, "H323\tFindCapability: " << mainType << " nonStandard");
+  PTRACE(4, "H323\tFindCapability: " << mainType << " nonStandard");
 
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     H323Capability & capability = table[i];
     if (capability.GetMainType() == mainType &&
         capability.GetSubType() == nonStandardTag &&
         capability.IsNonStandardMatch((const H245_NonStandardParameter &)subTypePDU.GetObject())) {
-      PTRACE(2, "H323\tFound capability: " << capability);
+      PTRACE(3, "H323\tFound capability: " << capability);
       return &capability;
     }
   }
@@ -1805,13 +1936,13 @@ H323Capability * H323Capabilities::FindCapability(H323Capability::MainTypes main
 H323Capability * H323Capabilities::FindCapability(H323Capability::MainTypes mainType,
                                                   unsigned subType) const
 {
-  PTRACE(3, "H323\tFindCapability: " << mainType << " subtype=" << subType);
+  PTRACE(4, "H323\tFindCapability: " << mainType << " subtype=" << subType);
 
   for (PINDEX i = 0; i < table.GetSize(); i++) {
     H323Capability & capability = table[i];
     if (capability.GetMainType() == mainType &&
                         (subType == UINT_MAX || capability.GetSubType() == subType)) {
-      PTRACE(2, "H323\tFound capability: " << capability);
+      PTRACE(3, "H323\tFound capability: " << capability);
       return &capability;
     }
   }
@@ -1820,7 +1951,8 @@ H323Capability * H323Capabilities::FindCapability(H323Capability::MainTypes main
 }
 
 
-void H323Capabilities::BuildPDU(H245_TerminalCapabilitySet & pdu) const
+void H323Capabilities::BuildPDU(const H323Connection & connection,
+                                H245_TerminalCapabilitySet & pdu) const
 {
   PINDEX tableSize = table.GetSize();
   PINDEX setSize = set.GetSize();
@@ -1831,11 +1963,16 @@ void H323Capabilities::BuildPDU(H245_TerminalCapabilitySet & pdu) const
   // Set the table of capabilities
   pdu.IncludeOptionalField(H245_TerminalCapabilitySet::e_capabilityTable);
 
-  pdu.m_capabilityTable.SetSize(tableSize);
+  PINDEX count = 0;
   for (PINDEX i = 0; i < tableSize; i++) {
-    pdu.m_capabilityTable[i].m_capabilityTableEntryNumber = table[i].GetCapabilityNumber();
-    pdu.m_capabilityTable[i].IncludeOptionalField(H245_CapabilityTableEntry::e_capability);
-    table[i].OnSendingPDU(pdu.m_capabilityTable[i].m_capability);
+    H323Capability & capability = table[i];
+    if (capability.IsUsable(connection)) {
+      pdu.m_capabilityTable.SetSize(count+1);
+      H245_CapabilityTableEntry & entry = pdu.m_capabilityTable[count++];
+      entry.m_capabilityTableEntryNumber = capability.GetCapabilityNumber();
+      entry.IncludeOptionalField(H245_CapabilityTableEntry::e_capability);
+      capability.OnSendingPDU(entry.m_capability);
+    }
   }
 
   // Set the sets of compatible capabilities
@@ -1852,8 +1989,14 @@ void H323Capabilities::BuildPDU(H245_TerminalCapabilitySet & pdu) const
       H245_AlternativeCapabilitySet & alt = desc.m_simultaneousCapabilities[middle];
       PINDEX innerSize = set[outer][middle].GetSize();
       alt.SetSize(innerSize);
-      for (PINDEX inner = 0; inner < innerSize; inner++)
-        alt[inner] = set[outer][middle][inner].GetCapabilityNumber();
+      count = 0;
+      for (PINDEX inner = 0; inner < innerSize; inner++) {
+        H323Capability & capability = set[outer][middle][inner];
+        if (capability.IsUsable(connection)) {
+          alt.SetSize(count+1);
+          alt[count++] = capability.GetCapabilityNumber();
+        }
+      }
     }
   }
 }
@@ -2006,16 +2149,128 @@ BOOL H323Capabilities::IsAllowed(const unsigned a_capno1, const unsigned a_capno
 H323CapabilityRegistration::H323CapabilityRegistration(const char * name)
   : PCaselessString(name)
 {
-  H323CapabilityRegistration * test = RegisteredCapabilitiesListHead;
+  PWaitAndSignal mutex(GetMutex());
+  H323CapabilityRegistration * test = registeredCapabilitiesListHead;
   while (test != NULL) {
     if (*test == *this)
       return;
     test = test->link;
   }
 
-  link = RegisteredCapabilitiesListHead;
-  RegisteredCapabilitiesListHead = this;
+  link = registeredCapabilitiesListHead;
+  registeredCapabilitiesListHead = this;
+}
+
+
+PMutex & H323CapabilityRegistration::GetMutex()
+{
+  static PMutex mutex;
+  return mutex;
 }
 
 
 /////////////////////////////////////////////////////////////////////////////
+
+#ifndef PASN_NOPRINTON
+
+
+struct msNonStandardCodecDef {
+  char * name;
+  BYTE sig[2];
+};
+
+
+static msNonStandardCodecDef msNonStandardCodec[] = {
+  { "L&H CELP 4.8k", { 0x01, 0x11 } },
+  { "ADPCM",         { 0x02, 0x00 } },
+  { "L&H CELP 8k",   { 0x02, 0x11 } },
+  { "L&H CELP 12k",  { 0x03, 0x11 } },
+  { "L&H CELP 16k",  { 0x04, 0x11 } },
+  { "IMA-ADPCM",     { 0x11, 0x00 } },
+  { "GSM",           { 0x31, 0x00 } },
+  { NULL,            { 0,    0    } }
+};
+
+void H245_AudioCapability::PrintOn(ostream & strm) const
+{
+  strm << GetTagName();
+
+  // tag 0 is nonstandard
+  if (tag == 0) {
+
+    H245_NonStandardParameter & param = (H245_NonStandardParameter &)GetObject();
+    const PBYTEArray & data = param.m_data;
+
+    switch (param.m_nonStandardIdentifier.GetTag()) {
+      case H245_NonStandardIdentifier::e_h221NonStandard:
+        {
+          H245_NonStandardIdentifier_h221NonStandard & h221 = param.m_nonStandardIdentifier;
+
+          // Microsoft is 181/0/21324
+          if ((h221.m_t35CountryCode   == 181) &&
+              (h221.m_t35Extension     == 0) &&
+              (h221.m_manufacturerCode == 21324)
+            ) {
+            PString name = "Unknown";
+            PINDEX i;
+            if (data.GetSize() >= 21) {
+              for (i = 0; msNonStandardCodec[i].name != NULL; i++) {
+                if ((data[20] == msNonStandardCodec[i].sig[0]) && 
+                    (data[21] == msNonStandardCodec[i].sig[1])) {
+                  name = msNonStandardCodec[i].name;
+                  break;
+                }
+              }
+            }
+            strm << (PString(" [Microsoft") & name) << "]";
+          }
+
+          // Equivalence is 9/0/61
+          else if ((h221.m_t35CountryCode   == 9) &&
+                   (h221.m_t35Extension     == 0) &&
+                   (h221.m_manufacturerCode == 61)
+                  ) {
+            PString name;
+            if (data.GetSize() > 0)
+              name = PString((const char *)(const BYTE *)data, data.GetSize());
+            strm << " [Equivalence " << name << "]";
+          }
+
+          // Xiph is 181/0/38
+          else if ((h221.m_t35CountryCode   == 181) &&
+                   (h221.m_t35Extension     == 0) &&
+                   (h221.m_manufacturerCode == 38)
+                  ) {
+            PString name;
+            if (data.GetSize() > 0)
+              name = PString((const char *)(const BYTE *)data, data.GetSize());
+            strm << " [Xiph " << name << "]";
+          }
+
+          // Cisco is 181/0/18
+          else if ((h221.m_t35CountryCode   == 181) &&
+                   (h221.m_t35Extension     == 0) &&
+                   (h221.m_manufacturerCode == 18)
+                  ) {
+            PString name;
+            if (data.GetSize() > 0)
+              name = PString((const char *)(const BYTE *)data, data.GetSize());
+            strm << " [Cisco " << name << "]";
+          }
+
+        }
+        break;
+      default:
+        break;
+    }
+  }
+
+  if (choice == NULL)
+    strm << " (NULL)";
+  else {
+    strm << ' ' << *choice;
+  }
+
+  //PASN_Choice::PrintOn(strm);
+}
+#endif
