@@ -27,6 +27,167 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: h323ep.cxx,v $
+ * Revision 1.166  2003/04/28 09:01:15  robertj
+ * Fixed problem with backward compatibility with non-url based remote
+ *   addresses passed to MakeCall()
+ *
+ * Revision 1.165  2003/04/24 01:49:33  dereks
+ * Add ability to set no media timeout interval
+ *
+ * Revision 1.164  2003/04/10 09:41:26  robertj
+ * Added associated transport to new GetInterfaceAddresses() function so
+ *   interfaces can be ordered according to active transport links. Improves
+ *   interoperability.
+ *
+ * Revision 1.163  2003/04/10 01:01:56  craigs
+ * Added functions to access to lists of interfaces
+ *
+ * Revision 1.162  2003/04/07 13:09:30  robertj
+ * Added ILS support to callto URL parsing in MakeCall(), ie can now call hosts
+ *   registered with an ILS directory.
+ *
+ * Revision 1.161  2003/04/07 11:11:45  craigs
+ * Fixed compile problem on Linux
+ *
+ * Revision 1.160  2003/04/04 08:04:35  robertj
+ * Added support for URL's in MakeCall, especially h323 and callto schemes.
+ *
+ * Revision 1.159  2003/03/04 03:58:32  robertj
+ * Fixed missing local interface usage if specified in UseGatekeeper()
+ *
+ * Revision 1.158  2003/02/28 09:00:37  rogerh
+ * remove redundant code
+ *
+ * Revision 1.157  2003/02/25 23:51:49  robertj
+ * Fxied bug where not getting last port in range, thanks Sonya Cooper-Hull
+ *
+ * Revision 1.156  2003/02/09 00:48:09  robertj
+ * Added function to return if registered with gatekeeper.
+ *
+ * Revision 1.155  2003/02/05 06:32:10  robertj
+ * Fixed non-stun symmetric NAT support recently broken.
+ *
+ * Revision 1.154  2003/02/05 04:56:35  robertj
+ * Fixed setting STUN server to enpty string clearing stun variable.
+ *
+ * Revision 1.153  2003/02/04 07:06:41  robertj
+ * Added STUN support.
+ *
+ * Revision 1.152  2003/02/01 13:31:22  robertj
+ * Changes to support CAT authentication in RAS.
+ *
+ * Revision 1.151  2003/01/26 05:57:29  robertj
+ * Changed ParsePartyName so will accept addresses of the form
+ *   alias@gk:address which will do an LRQ call to "address" using "alias"
+ *   to determine the IP address to connect to.
+ *
+ * Revision 1.150  2003/01/23 02:36:32  robertj
+ * Increased (and made configurable) timeout for H.245 channel TCP connection.
+ *
+ * Revision 1.149  2003/01/06 06:13:37  robertj
+ * Increased maximum possible jitter configuration to 10 seconds.
+ *
+ * Revision 1.148  2002/11/27 06:54:57  robertj
+ * Added Service Control Session management as per Annex K/H.323 via RAS
+ *   only at this stage.
+ * Added H.248 ASN and very primitive infrastructure for linking into the
+ *   Service Control Session management system.
+ * Added basic infrastructure for Annex K/H.323 HTTP transport system.
+ * Added Call Credit Service Control to display account balances.
+ *
+ * Revision 1.147  2002/11/19 07:07:44  robertj
+ * Changed priority so H.235 standard authentication used by preference.
+ *
+ * Revision 1.146  2002/11/15 06:53:24  robertj
+ * Fixed non facility redirect calls being able to be cleared!
+ *
+ * Revision 1.145  2002/11/15 05:17:26  robertj
+ * Added facility redirect support without changing the call token for access
+ *   to the call. If it gets redirected a new H323Connection object is
+ *   created but it looks like the same thing to an application.
+ *
+ * Revision 1.144  2002/11/14 22:06:08  robertj
+ * Increased default maximum number of ports.
+ *
+ * Revision 1.143  2002/11/10 08:10:43  robertj
+ * Moved constants for "well known" ports to better place (OPAL change).
+ *
+ * Revision 1.142  2002/10/31 00:42:41  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.141  2002/10/24 07:18:24  robertj
+ * Changed gatekeeper call so if can do local DNS lookup or using IP address
+ *   then uses destCallSignalAddress for ARQ instead of destinationInfo.
+ *
+ * Revision 1.140  2002/10/23 06:06:13  robertj
+ * Added function to be smarter in using a gatekeeper for use by endpoint.
+ *
+ * Revision 1.139  2002/10/01 06:38:36  robertj
+ * Removed GNU compiler warning
+ *
+ * Revision 1.138  2002/10/01 03:07:15  robertj
+ * Added version number functions for OpenH323 library itself, plus included
+ *   library version in the default vendor information.
+ *
+ * Revision 1.137  2002/08/15 04:56:56  robertj
+ * Fixed operation of ports system assuring RTP is even numbers.
+ *
+ * Revision 1.136  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
+ * Revision 1.135  2002/07/19 11:25:10  robertj
+ * Added extra trace on attempt to clear non existent call.
+ *
+ * Revision 1.134  2002/07/19 03:39:22  robertj
+ * Bullet proofed setting of RTP IP port base, can't be zero!
+ *
+ * Revision 1.133  2002/07/18 01:50:14  robertj
+ * Changed port secltion code to force apps to use function interface.
+ *
+ * Revision 1.132  2002/07/04 00:11:25  robertj
+ * Fixed setting of gk password when password changed in endpoint.
+ *
+ * Revision 1.131  2002/06/22 05:48:42  robertj
+ * Added partial implementation for H.450.11 Call Intrusion
+ *
+ * Revision 1.130  2002/06/15 03:06:46  robertj
+ * Fixed locking of connection on H.250.2 transfer, thanks Gilles Delcayre
+ *
+ * Revision 1.129  2002/06/13 06:15:20  robertj
+ * Allowed TransferCall() to be used on H323Connection as well as H323EndPoint.
+ *
+ * Revision 1.128  2002/06/12 03:55:21  robertj
+ * Added function to add/remove multiple listeners in one go comparing against
+ *   what is already running so does not interrupt unchanged listeners.
+ *
+ * Revision 1.127  2002/05/29 06:40:33  robertj
+ * Changed sending of endSession/ReleaseComplete PDU's to occur immediately
+ *   on call clearance and not wait for background thread to do it.
+ * Stricter compliance by waiting for reply endSession before closing down.
+ *
+ * Revision 1.126  2002/05/28 06:16:12  robertj
+ * Split UDP (for RAS) from RTP port bases.
+ * Added current port variable so cycles around the port range specified which
+ *   fixes some wierd problems on some platforms, thanks Federico Pinna
+ *
+ * Revision 1.125  2002/05/17 03:39:01  robertj
+ * Fixed problems with H.235 authentication on RAS for server and client.
+ *
+ * Revision 1.124  2002/05/15 23:57:49  robertj
+ * Added function to get the tokens for all active calls.
+ * Fixed setting of password info in gatekeeper so is before discovery.
+ *
+ * Revision 1.123  2002/05/02 07:56:28  robertj
+ * Added automatic clearing of call if no media (RTP data) is transferred in a
+ *   configurable (default 5 minutes) amount of time.
+ *
+ * Revision 1.122  2002/04/18 01:40:56  robertj
+ * Fixed bad variable name for disabling DTMF detection, very confusing.
+ *
+ * Revision 1.121  2002/04/17 00:50:57  robertj
+ * Added ability to disable the in band DTMF detection.
+ *
  * Revision 1.120  2002/04/10 08:09:03  robertj
  * Allowed zero TCP ports
  *
@@ -419,17 +580,23 @@
  *
  */
 
+#include <ptlib.h>
+
 #ifdef __GNUC__
 #pragma implementation "h323ep.h"
 #endif
 
-#include <ptlib.h>
-#include <ptclib/random.h>
 #include "h323ep.h"
 #include "h323pdu.h"
 #include "h450pdu.h"
 #include "gkclient.h"
 #include "t38proto.h"
+#include "../version.h"
+
+#include <ptclib/random.h>
+#include <ptclib/pstun.h>
+#include <ptclib/url.h>
+#include <ptclib/pils.h>
 
 
 #ifndef IPTOS_PREC_CRITIC_ECP
@@ -490,6 +657,35 @@ class H323ConnectionsCleaner : public PThread
 #endif
 
 
+/////////////////////////////////////////////////////////////////////////////
+
+PString OpalGetVersion()
+{
+#define AlphaCode   "alpha"
+#define BetaCode    "beta"
+#define ReleaseCode "."
+
+  return psprintf("%u.%u%s%u", MAJOR_VERSION, MINOR_VERSION, BUILD_TYPE, BUILD_NUMBER);
+}
+
+
+unsigned OpalGetMajorVersion()
+{
+  return MAJOR_VERSION;
+}
+
+unsigned OpalGetMinorVersion()
+{
+  return MINOR_VERSION;
+}
+
+unsigned OpalGetBuildNumber()
+{
+  return BUILD_NUMBER;
+}
+
+
+
 /////////////////////////////////////////////////////////////////////////////
 
 H225CallThread::H225CallThread(H323EndPoint & endpoint,
@@ -575,18 +771,27 @@ H323EndPoint::H323EndPoint()
   : soundChannelPlayDevice(PSoundChannel::GetDefaultDevice(PSoundChannel::Player)),
     soundChannelRecordDevice(PSoundChannel::GetDefaultDevice(PSoundChannel::Recorder)),
     signallingChannelCallTimeout(0, 0, 1),  // Minutes
+    controlChannelStartTimeout(0, 0, 2),    // Minutes
+    endSessionTimeout(0, 10),               // Seconds
     masterSlaveDeterminationTimeout(0, 30), // Seconds
     capabilityExchangeTimeout(0, 30),       // Seconds
     logicalChannelTimeout(0, 30),           // Seconds
     requestModeTimeout(0, 30),              // Seconds
     roundTripDelayTimeout(0, 10),           // Seconds
     roundTripDelayRate(0, 0, 1),            // Minutes
+    noMediaTimeout(0, 0, 5),                // Minutes
     gatekeeperRequestTimeout(0, 5),         // Seconds
     rasRequestTimeout(0, 3),                // Seconds
     callTransferT1(0,10),                   // Seconds
     callTransferT2(0,10),                   // Seconds
     callTransferT3(0,10),                   // Seconds
-    callTransferT4(0,10)                    // Seconds
+    callTransferT4(0,10),                   // Seconds
+    callIntrusionT1(0,30),                  // Seconds
+    callIntrusionT2(0,30),                  // Seconds
+    callIntrusionT3(0,30),                  // Seconds
+    callIntrusionT4(0,30),                  // Seconds
+    callIntrusionT5(0,10),                  // Seconds
+    callIntrusionT6(0,10)                   // Seconds
 {
   PString username = PProcess::Current().GetUserName();
   if (username.IsEmpty())
@@ -595,14 +800,20 @@ H323EndPoint::H323EndPoint()
 
   autoStartReceiveVideo = autoStartTransmitVideo = TRUE;
   autoStartReceiveFax = autoStartTransmitFax = FALSE;
+  autoCallForward = TRUE;
   disableFastStart = FALSE;
   disableH245Tunneling = FALSE;
   disableH245inSetup = FALSE;
+  disableDetectInBandDTMF = FALSE;
+  canDisplayAmountString = FALSE;
+  canEnforceDurationLimit = TRUE;
+  callIntrusionProtectionLevel = 3; //H45011_CIProtectionLevel::e_fullProtection;
   defaultSilenceDetection = H323AudioCodec::AdaptiveSilenceDetection;
   defaultSendUserInputMode = H323Connection::SendUserInputAsString;
 
   terminalType = e_TerminalOnly;
-  maxAudioDelayJitter = 50; // milliseconds
+  minAudioJitterDelay = 50;  // milliseconds
+  maxAudioJitterDelay = 250; // milliseconds
   initialBandwidth = 100000; // Standard 10base LAN in 100's of bits/sec
   clearCallOnRoundTripFail = FALSE;
 
@@ -610,11 +821,14 @@ H323EndPoint::H323EndPoint()
   t35Extension = 0;
   manufacturerCode = 61; // Allocated by Australian Communications Authority, Oct 2000
 
-  rtpIpPortBase = 5000;
-  rtpIpPortMax  = 6000;
+  rtpIpPorts.current = rtpIpPorts.base = 5000;
+  rtpIpPorts.max = 5999;
 
   // use dynamic port allocation by default
-  tcpPortBase   = tcpPortMax = 0;
+  tcpPorts.current = tcpPorts.base = tcpPorts.max = 0;
+  udpPorts.current = udpPorts.base = udpPorts.max = 0;
+
+  stun = NULL;
 
 #ifdef _WIN32
   // Windows MultMedia stuff seems to need greater depth due to enormous
@@ -668,6 +882,8 @@ H323EndPoint::~H323EndPoint()
   // Clean up any connections that the cleaner thread missed
   CleanUpConnections();
 
+  delete stun;
+
   PTRACE(3, "H323\tDeleted endpoint.");
 }
 
@@ -714,7 +930,7 @@ void H323EndPoint::SetVendorIdentifierInfo(H225_VendorIdentifier & info) const
   info.m_productId.SetSize(info.m_productId.GetSize()+2);
 
   info.IncludeOptionalField(H225_VendorIdentifier::e_versionId);
-  info.m_versionId = PProcess::Current().GetVersion(TRUE);
+  info.m_versionId = PProcess::Current().GetVersion(TRUE) + " (OpenH323 v" + OpalGetVersion() + ')';
   info.m_versionId.SetSize(info.m_versionId.GetSize()+2);
 }
 
@@ -787,6 +1003,52 @@ void H323EndPoint::ReorderCapabilities(const PStringArray & preferenceOrder)
 }
 
 
+BOOL H323EndPoint::UseGatekeeper(const PString & address,
+                                 const PString & identifier,
+                                 const PString & localAddress)
+{
+  if (gatekeeper != NULL) {
+    BOOL same = TRUE;
+
+    if (!address)
+      same = gatekeeper->GetTransport().GetRemoteAddress().IsEquivalent(address);
+
+    if (!same && !identifier)
+      same = gatekeeper->GetIdentifier() == identifier;
+
+    if (!same && !localAddress)
+      same = gatekeeper->GetTransport().GetLocalAddress().IsEquivalent(localAddress);
+
+    if (same) {
+      PTRACE(2, "H323\tUsing existing gatekeeper " << *gatekeeper);
+      return TRUE;
+    }
+  }
+
+  H323Transport * transport = NULL;
+  if (!localAddress.IsEmpty()) {
+    H323TransportAddress iface(localAddress);
+    PIPSocket::Address ip;
+    WORD port = H225_RAS::DefaultRasUdpPort;
+    if (iface.GetIpAndPort(ip, port))
+      transport = new H323TransportUDP(*this, ip, port);
+  }
+
+  if (address.IsEmpty()) {
+    if (identifier.IsEmpty())
+      return DiscoverGatekeeper(transport);
+    else
+      return LocateGatekeeper(identifier, transport);
+  }
+  else {
+    if (identifier.IsEmpty())
+      return SetGatekeeper(address, transport);
+    else
+      return SetGatekeeperZone(address, identifier, transport);
+  }
+}
+
+
 BOOL H323EndPoint::SetGatekeeper(const PString & address, H323Transport * transport)
 {
   H323Gatekeeper * gk = InternalCreateGatekeeper(transport);
@@ -824,20 +1086,23 @@ H323Gatekeeper * H323EndPoint::InternalCreateGatekeeper(H323Transport * transpor
   if (transport == NULL)
     transport = new H323TransportUDP(*this);
 
-  return CreateGatekeeper(transport);
+  H323Gatekeeper * gk = CreateGatekeeper(transport);
+
+  gk->SetPassword(gatekeeperPassword);
+
+  return gk;
 }
 
 
 BOOL H323EndPoint::InternalRegisterGatekeeper(H323Gatekeeper * gk, BOOL discovered)
 {
   if (discovered) {
-    gk->SetPassword(gatekeeperPassword);
     if (gk->RegistrationRequest()) {
       gatekeeper = gk;
       return TRUE;
     }
 
-    // RRQ was rejected continue listening
+    // RRQ was rejected continue trying
     gatekeeper = gk;
   }
   else // Only stop listening if the GRQ was rejected
@@ -853,6 +1118,15 @@ H323Gatekeeper * H323EndPoint::CreateGatekeeper(H323Transport * transport)
 }
 
 
+BOOL H323EndPoint::IsRegisteredWithGatekeeper() const
+{
+  if (gatekeeper == NULL)
+    return FALSE;
+
+  return gatekeeper->IsRegistered();
+}
+
+
 BOOL H323EndPoint::RemoveGatekeeper(int reason)
 {
   BOOL ok = TRUE;
@@ -872,12 +1146,69 @@ BOOL H323EndPoint::RemoveGatekeeper(int reason)
 }
 
 
+void H323EndPoint::SetGatekeeperPassword(const PString & password)
+{
+  gatekeeperPassword = password;
+
+  if (gatekeeper != NULL) {
+    gatekeeper->SetPassword(gatekeeperPassword);
+    if (gatekeeper->IsRegistered()) // If we are registered send a URQ
+      gatekeeper->UnregistrationRequest(H225_UnregRequestReason::e_reregistrationRequired);
+    InternalRegisterGatekeeper(gatekeeper, TRUE);
+  }
+}
+
+
+H235Authenticators H323EndPoint::CreateAuthenticators()
+{
+  H235Authenticators authenticators;
+
+#if P_SSL
+  authenticators.Append(new H235AuthProcedure1);
+#endif
+  authenticators.Append(new H235AuthSimpleMD5);
+  authenticators.Append(new H235AuthCAT);
+
+  return authenticators;
+}
+
+
+BOOL H323EndPoint::StartListeners(const H323TransportAddressArray & ifaces)
+{
+  if (ifaces.IsEmpty())
+    return StartListener("*");
+
+  PINDEX i;
+
+  for (i = 0; i < listeners.GetSize(); i++) {
+    BOOL remove = TRUE;
+    for (PINDEX j = 0; j < ifaces.GetSize(); j++) {
+      if (listeners[i].GetTransportAddress().IsEquivalent(ifaces[j])) {
+        remove = FALSE;
+        break;
+      }
+    }
+    if (remove) {
+      PTRACE(3, "H323\tRemoving listener " << listeners[i]);
+      listeners.RemoveAt(i--);
+    }
+  }
+
+  for (i = 0; i < ifaces.GetSize(); i++) {
+    if (!ifaces[i])
+      StartListener(ifaces[i]);
+  }
+
+  return listeners.GetSize() > 0;
+}
+
+
 BOOL H323EndPoint::StartListener(const H323TransportAddress & iface)
 {
   H323Listener * listener;
 
   if (iface.IsEmpty())
-    listener = new H323ListenerTCP(*this);
+    listener = new H323ListenerTCP(*this, INADDR_ANY, DefaultTcpPort);
   else
     listener = iface.CreateListener(*this);
 
@@ -892,6 +1223,17 @@ BOOL H323EndPoint::StartListener(const H323TransportAddress & iface)
 
 BOOL H323EndPoint::StartListener(H323Listener * listener)
 {
+  if (listener == NULL)
+    return FALSE;
+
+  for (PINDEX i = 0; i < listeners.GetSize(); i++) {
+    if (listeners[i].GetTransportAddress() == listener->GetTransportAddress()) {
+      PTRACE(2, "H323\tAlready have listener for " << *listener);
+      delete listener;
+      return TRUE;
+    }
+  }
+
   // as the listener is not open, this will have the effect of immediately
   // stopping the listener thread. This is good - it means that the 
   // listener Close function will appear to have stopped the thread
@@ -900,6 +1242,7 @@ BOOL H323EndPoint::StartListener(H323Listener * listener)
     return FALSE;
   }
 
+  PTRACE(3, "H323\tStarted listener " << *listener);
   listeners.Append(listener);
   listener->Resume();
   return TRUE;
@@ -908,23 +1251,21 @@ BOOL H323EndPoint::StartListener(H323Listener * listener)
 
 BOOL H323EndPoint::RemoveListener(H323Listener * listener)
 {
-  if (listener != NULL)
+  if (listener != NULL) {
+    PTRACE(3, "H323\tRemoving listener " << *listener);
     return listeners.Remove(listener);
+  }
 
+  PTRACE(3, "H323\tRemoving all listeners");
   listeners.RemoveAll();
   return TRUE;
 }
 
 
-void H323EndPoint::SetGatekeeperPassword(const PString & password)
+H323TransportAddressArray H323EndPoint::GetInterfaceAddresses(BOOL excludeLocalHost,
+                                                              H323Transport * associatedTransport)
 {
-  gatekeeperPassword = password;
-
-  if (gatekeeper != NULL) {
-    if (gatekeeper->IsRegistered()) // If we are registered send a URQ
-      gatekeeper->UnregistrationRequest(H225_UnregRequestReason::e_reregistrationRequired);
-    InternalRegisterGatekeeper(gatekeeper, TRUE);
-  }
+  return H323GetInterfaceAddresses(listeners, excludeLocalHost, associatedTransport);
 }
 
 
@@ -941,9 +1282,14 @@ H323Connection * H323EndPoint::MakeCall(const PString & remoteParty,
                                         PString & token,
                                         void * userData)
 {
-  H323Connection * connection = InternalMakeCall(PString(), PString(),
-                                                 remoteParty, transport,
-                                                 token, userData);
+  token = PString::Empty();
+  H323Connection * connection = InternalMakeCall(PString::Empty(),
+                                                 PString::Empty(),
+                                                 UINT_MAX,
+                                                 remoteParty,
+                                                 transport,
+                                                 token,
+                                                 userData);
   if (connection != NULL)
     connection->Unlock();
   return connection;
@@ -955,12 +1301,20 @@ H323Connection * H323EndPoint::MakeCallLocked(const PString & remoteParty,
                                               void * userData,
                                               H323Transport * transport)
 {
-  return InternalMakeCall(PString(), PString(), remoteParty, transport, token, userData);
+  token = PString::Empty();
+  return InternalMakeCall(PString::Empty(),
+                          PString::Empty(),
+                          UINT_MAX,
+                          remoteParty,
+                          transport,
+                          token,
+                          userData);
 }
 
 
-H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
+H323Connection * H323EndPoint::InternalMakeCall(const PString & trasferFromToken,
                                                 const PString & callIdentity,
+                                                unsigned capabilityLevel,
                                                 const PString & remoteParty,
                                                 H323Transport * transport,
                                                 PString & newToken,
@@ -970,7 +1324,10 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
 
   PString alias;
   H323TransportAddress address;
-  ParsePartyName(remoteParty, alias, address);
+  if (!ParsePartyName(remoteParty, alias, address)) {
+    PTRACE(2, "H323\tCould not parse \"" << remoteParty << '"');
+    return NULL;
+  }
 
   if (transport == NULL) {
     // Restriction: the call must be made on the same transport as the one
@@ -985,15 +1342,33 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
     }
   }
 
+  H323Connection * connection;
+
   connectionsMutex.Wait();
 
   unsigned lastReference;
-  do {
-    lastReference = Q931::GenerateCallReference();
-    newToken = BuildConnectionToken(*transport, lastReference, FALSE);
-  } while (connectionsActive.Contains(newToken));
+  if (newToken.IsEmpty()) {
+    do {
+      lastReference = Q931::GenerateCallReference();
+      newToken = BuildConnectionToken(*transport, lastReference, FALSE);
+    } while (connectionsActive.Contains(newToken));
+  }
+  else {
+    lastReference = newToken.Mid(newToken.Find('/')+1).AsUnsigned();
 
-  H323Connection * connection = CreateConnection(lastReference, userData, transport, NULL);
+    // Move old connection on token to new value and flag for removal
+    PString adjustedToken;
+    unsigned tieBreaker = 0;
+    do {
+      adjustedToken = newToken + "-replaced";
+      adjustedToken.sprintf("-%u", ++tieBreaker);
+    } while (connectionsActive.Contains(adjustedToken));
+    connectionsActive.SetAt(adjustedToken, connectionsActive.RemoveAt(newToken));
+    connectionsToBeCleaned += adjustedToken;
+    PTRACE(3, "H323\tOverwriting call " << newToken << ", renamed to " << adjustedToken);
+  }
+
+  connection = CreateConnection(lastReference, userData, transport, NULL);
   if (connection == NULL) {
     PTRACE(1, "H323\tCreateConnection returned NULL");
     connectionsMutex.Signal();
@@ -1006,8 +1381,14 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
 
   connectionsMutex.Signal();
 
-  connection->AttachSignalChannel(transport, FALSE);
-  connection->HandleTransferCall(existingToken, callIdentity);
+  connection->AttachSignalChannel(newToken, transport, FALSE);
+
+  if (capabilityLevel == UINT_MAX)
+    connection->HandleTransferCall(trasferFromToken, callIdentity);
+  else {
+    connection->HandleIntrudeCall(trasferFromToken, callIdentity);
+    connection->IntrudeCall(capabilityLevel);
+  }
 
   PTRACE(3, "H323\tCreated new connection: " << newToken);
 
@@ -1016,27 +1397,163 @@ H323Connection * H323EndPoint::InternalMakeCall(const PString & existingToken,
 }
 
 
-void H323EndPoint::ParsePartyName(const PString & remoteParty,
+BOOL H323EndPoint::ParsePartyName(const PString & remoteParty,
                                   PString & alias,
-                                  H323TransportAddress & address) const
+                                  H323TransportAddress & address)
 {
-  /*Determine the alias part and the address part of the remote party name
-    string. This depends on if there is an '@' to separate the alias from the
-    transport address or a '$' for if it is explicitly a transport address.
-   */
-  PINDEX at = remoteParty.FindLast('@');
-  if (at != P_MAX_INDEX) {
-    alias = remoteParty.Left(at++);
-    address = remoteParty.Mid(at);
+  PURL url(remoteParty, "h323");
+
+  // Special adjustment if 
+  if (remoteParty.Find('@') == P_MAX_INDEX &&
+      remoteParty.NumCompare(url.GetScheme()) != EqualTo) {
+    if (gatekeeper == NULL)
+      url.Parse("h323:@" + remoteParty);
+    else
+      url.Parse("h323:" + remoteParty);
   }
-  else if (gatekeeper != NULL) {
-    alias = remoteParty;
-    address = H323TransportAddress();
+
+  alias = url.GetUserName();
+
+  address = url.GetHostName();
+  if (!address && url.GetPort() != 0)
+    address.sprintf(":%u", url.GetPort());
+
+  if (alias.IsEmpty() && address.IsEmpty()) {
+    PTRACE(1, "H323\tAttempt to use invalid URL \"" << remoteParty << '"');
+    return FALSE;
   }
-  else {
-    alias = "";
-    address = remoteParty;
+
+  BOOL gatekeeperSpecified = FALSE;
+  BOOL gatewaySpecified = FALSE;
+
+  PCaselessString type = url.GetParamVars()("type");
+
+  if (url.GetScheme() == "callto") {
+    // Do not yet support ILS
+    if (type == "directory") {
+#if P_LDAP
+      PString server = url.GetHostName();
+      if (server.IsEmpty())
+        server = ilsServer;
+      if (server.IsEmpty())
+        return FALSE;
+
+      PILSSession ils;
+      if (!ils.Open(server, url.GetPort())) {
+        PTRACE(1, "H323\tCould not open ILS server at \"" << server
+               << "\" - " << ils.GetErrorText());
+        return FALSE;
+      }
+
+      PILSSession::RTPerson person;
+      if (!ils.SearchPerson(alias, person)) {
+        PTRACE(1, "H323\tCould not find "
+               << server << '/' << alias << ": " << ils.GetErrorText());
+        return FALSE;
+      }
+
+      if (!person.sipAddress.IsValid()) {
+        PTRACE(1, "H323\tILS user " << server << '/' << alias
+               << " does not have a valid IP address");
+        return FALSE;
+      }
+
+      // Get the IP address
+      address = H323TransportAddress(person.sipAddress);
+
+      // Get the port if provided
+      for (PINDEX i = 0; i < person.sport.GetSize(); i++) {
+        if (person.sport[i] != 1503) { // Dont use the T.120 port
+          address = H323TransportAddress(person.sipAddress, person.sport[i]);
+          break;
+        }
+      }
+
+      alias = PString::Empty(); // No alias for ILS lookup, only host
+      return TRUE;
+#else
+      return FALSE;
+#endif
+    }
+
+    if (url.GetParamVars().Contains("gateway"))
+      gatewaySpecified = TRUE;
   }
+  else if (url.GetScheme() == "h323") {
+    if (type == "gw")
+      gatewaySpecified = TRUE;
+    else if (type == "gk")
+      gatekeeperSpecified = TRUE;
+    else if (!type) {
+      PTRACE(1, "H323\tUnsupported host type \"" << type << "\" in h323 URL");
+      return FALSE;
+    }
+  }
+
+  // User explicitly asked fo a look up to a gk
+  if (gatekeeperSpecified) {
+    if (alias.IsEmpty()) {
+      PTRACE(1, "H323\tAttempt to use explict gatekeeper without alias!");
+      return FALSE;
+    }
+
+    if (address.IsEmpty()) {
+      PTRACE(1, "H323\tAttempt to use explict gatekeeper without address!");
+      return FALSE;
+    }
+
+    H323TransportAddress gkAddr = address;
+    PTRACE(3, "H323\tLooking for \"" << alias << "\" on gatekeeper at " << gkAddr);
+
+    H323Gatekeeper * gk = CreateGatekeeper(new H323TransportUDP(*this));
+
+    BOOL ok = gk->DiscoverByAddress(gkAddr);
+    if (ok) {
+      ok = gk->LocationRequest(alias, address);
+      if (ok) {
+        PTRACE(3, "H323\tLocation Request of \"" << alias << "\" on gk " << gkAddr << " found " << address);
+      }
+      else {
+        PTRACE(1, "H323\tLocation Request failed for \"" << alias << "\" on gk " << gkAddr);
+      }
+    }
+    else {
+      PTRACE(1, "H323\tLocation Request discovery failed for gk " << gkAddr);
+    }
+
+    delete gk;
+
+    return ok;
+  }
+
+  // User explicitly said to use a gw, or we do not have a gk to do look up
+  if (gatekeeper == NULL || gatewaySpecified) {
+    // If URL did not have a host, but user said to use gw, or we do not have
+    // a gk to do a lookup so we MUST have a host, use alias must be host
+    if (address.IsEmpty()) {
+      address = alias;
+      alias = PString::Empty();
+    }
+    return TRUE;
+  }
+
+  // We have a gatekeeper and user provided a host, all done
+  if (!address)
+    return TRUE;
+
+  // We have a gk and user did not explicitly supply a host, so lets
+  // do a check to see if it is an IP address or hostname
+  if (alias.FindOneOf("$.:[") != P_MAX_INDEX) {
+    H323TransportAddress test = alias;
+    PIPSocket::Address ip;
+    if (test.GetIpAddress(ip) && ip.IsValid()) {
+      // The alias was a valid internet address, use it as such
+      alias = PString::Empty();
+      address = test;
+    }
+  }
+
+  return TRUE;
 }
 
 
@@ -1046,7 +1563,17 @@ H323Connection * H323EndPoint::SetupTransfer(const PString & oldToken,
                                              PString & newToken,
                                              void * userData)
 {
-  return InternalMakeCall(oldToken, callIdentity, remoteParty, NULL, newToken, userData);
+  newToken = PString::Empty();
+  H323Connection* connection = InternalMakeCall(oldToken,
+                                                callIdentity,
+                                                UINT_MAX,
+                                                remoteParty,
+                                                NULL,
+                                                newToken,
+                                                userData);
+  if (connection != NULL)
+     connection->Unlock();
+   return connection;
 }
 
 
@@ -1056,12 +1583,6 @@ void H323EndPoint::TransferCall(const PString & token,
 {
   H323Connection * connection = FindConnectionWithLock(token);
   if (connection != NULL) {
-
-    // According to H.450.4, if prior to consultation the primary call has been put on hold, the 
-    // transferring endpoint shall first retrieve the call before Call Transfer is invoked.
-    if (!callIdentity.IsEmpty() && connection->IsLocalHold()) {
-      connection->RetrieveCall();
-    }
     connection->TransferCall(remoteParty, callIdentity);
     connection->Unlock();
   }
@@ -1089,6 +1610,35 @@ void H323EndPoint::HoldCall(const PString & token, BOOL localHold)
 }
 
 
+H323Connection * H323EndPoint::IntrudeCall(const PString & remoteParty,
+                                        PString & token,
+                                        unsigned capabilityLevel,
+                                        void * userData)
+{
+  return IntrudeCall(remoteParty, NULL, token, capabilityLevel, userData);
+}
+
+
+H323Connection * H323EndPoint::IntrudeCall(const PString & remoteParty,
+                                        H323Transport * transport,
+                                        PString & token,
+                                        unsigned capabilityLevel,
+                                        void * userData)
+{
+  token = PString::Empty();
+  H323Connection * connection = InternalMakeCall(PString::Empty(),
+                                                 PString::Empty(),
+                                                 capabilityLevel,
+                                                 remoteParty,
+                                                 transport,
+                                                 token,
+                                                 userData);
+  if (connection != NULL)
+    connection->Unlock();
+  return connection;
+}
+
+
 BOOL H323EndPoint::ClearCall(const PString & token,
                              H323Connection::CallEndReason reason)
 {
@@ -1122,8 +1672,10 @@ BOOL H323EndPoint::ClearCallSynchronous(const PString & token,
 
     // Find the connection by token, callid or conferenceid
     H323Connection * connection = FindConnectionWithoutLocks(token);
-    if (connection == NULL)
+    if (connection == NULL) {
+      PTRACE(3, "H323\tAttempt to clear unknown call " << token);
       return FALSE;
+    }
 
     PTRACE(3, "H323\tClearing connection " << connection->GetCallToken()
                                            << " reason=" << reason);
@@ -1300,6 +1852,21 @@ H323Connection * H323EndPoint::FindConnectionWithoutLocks(const PString & token)
 }
 
 
+PStringList H323EndPoint::GetAllConnections()
+{
+  PStringList tokens;
+
+  connectionsMutex.Wait();
+
+  for (PINDEX i = 0; i < connectionsActive.GetSize(); i++)
+    tokens.AppendString(connectionsActive.GetKeyAt(i));
+
+  connectionsMutex.Signal();
+
+  return tokens;
+}
+
+
 BOOL H323EndPoint::OnIncomingCall(H323Connection & /*connection*/,
                                   const H323SignalPDU & /*setupPDU*/,
                                   H323SignalPDU & /*alertingPDU*/)
@@ -1349,6 +1916,28 @@ BOOL H323EndPoint::OnConnectionForwarded(H323Connection & /*connection*/,
 }
 
 
+BOOL H323EndPoint::ForwardConnection(H323Connection & connection,
+                                     const PString & forwardParty,
+                                     const H323SignalPDU & /*pdu*/)
+{
+  PString token = connection.GetCallToken();
+  H323Connection * newConnection = InternalMakeCall(PString::Empty(),
+                                                    PString::Empty(),
+                                                    UINT_MAX,
+                                                    forwardParty,
+                                                    NULL,
+                                                    token,
+                                                    NULL);
+  if (newConnection == NULL)
+    return FALSE;
+
+  connection.SetCallEndReason(H323Connection::EndedByCallForwarded);
+
+  newConnection->Unlock();
+  return TRUE;
+}
+
+
 void H323EndPoint::OnConnectionEstablished(H323Connection & /*connection*/,
                                            const PString & /*token*/)
 {
@@ -1414,7 +2003,7 @@ H323Connection * H323EndPoint::OnIncomingConnection(H323Transport * transport,
     connectionsMutex.Signal();
   }
 
-  connection->AttachSignalChannel(transport, TRUE);
+  connection->AttachSignalChannel(token, transport, TRUE);
 
   return connection;
 }
@@ -1555,6 +2144,41 @@ void H323EndPoint::OnUserInputTone(H323Connection & connection,
 }
 
 
+void H323EndPoint::OnHTTPServiceControl(unsigned /*opeartion*/,
+                                        unsigned /*sessionId*/,
+                                        const PString & /*url*/)
+{
+}
+
+
+void H323EndPoint::OnCallCreditServiceControl(const PString & /*amount*/, BOOL /*mode*/)
+{
+}
+
+
+void H323EndPoint::OnServiceControlSession(unsigned type,
+                                           unsigned sessionId,
+                                           const H323ServiceControlSession & session,
+                                           H323Connection * connection)
+{
+  session.OnChange(type, sessionId, *this, connection);
+}
+
+
+H323ServiceControlSession * H323EndPoint::CreateServiceControlSession(const H225_ServiceControlDescriptor & contents)
+{
+  switch (contents.GetTag()) {
+    case H225_ServiceControlDescriptor::e_url :
+      return new H323HTTPServiceControl(contents);
+
+    case H225_ServiceControlDescriptor::e_callCreditServiceControl :
+      return new H323CallCreditServiceControl(contents);
+  }
+
+  return NULL;
+}
+
+
 OpalT120Protocol * H323EndPoint::CreateT120ProtocolHandler(const H323Connection &) const
 {
   return NULL;
@@ -1691,39 +2315,168 @@ BOOL H323EndPoint::IsMCU() const
 }
 
 
-static void LimitPorts(unsigned & pBase, unsigned & pMax, unsigned range)
+void H323EndPoint::SetAudioJitterDelay(unsigned minDelay, unsigned maxDelay)
 {
-  if (pBase < 1024)
-    pBase = 1024;
-  else if (pBase > 65500)
-    pBase = 65500;
+  PAssert(minDelay <= 10000 && maxDelay <= 10000, PInvalidParameter);
 
-  if (pMax <= pBase)
-    pMax = pBase + range;
-  if (pMax > 65535)
-    pMax = 65535;
+  if (minDelay < 10)
+    minDelay = 10;
+  minAudioJitterDelay = minDelay;
+
+  if (maxDelay < minDelay)
+    maxDelay = minDelay;
+  maxAudioJitterDelay = maxDelay;
+}
+
+
+PSTUNClient * H323EndPoint::GetSTUN(const PIPSocket::Address & ip) const
+{
+  if (ip.IsValid() && IsLocalAddress(ip))
+    return NULL;
+
+  return stun;
+}
+
+
+void H323EndPoint::SetSTUNServer(const PString & server)
+{
+  delete stun;
+
+  if (server.IsEmpty())
+    stun = NULL;
+  else {
+    stun = new PSTUNClient(server,
+                           GetUDPPortBase(), GetUDPPortMax(),
+                           GetRtpIpPortBase(), GetRtpIpPortMax());
+    PTRACE(2, "H323\tSTUN server \"" << server << "\" replies " << stun->GetNatTypeName());
+  }
+}
+
+
+BOOL H323EndPoint::IsLocalAddress(const PIPSocket::Address & ip) const
+{
+  /* Check if the remote address is a private IP address.
+   * RFC 1918 specifies the following private IP addresses
+   * 10.0.0.0    - 10.255.255.255.255
+   * 172.16.0.0  - 172.31.255.255
+   * 192.168.0.0 - 192.168.255.255
+   */
+
+  return (ip.Byte1() == 10)
+         ||
+         (
+           (ip.Byte1() == 172)
+           &&
+           (ip.Byte2() >= 16) && (ip.Byte2() <= 31)
+         )
+         ||
+         (
+           (ip.Byte1() == 192) 
+           &&
+           (ip.Byte2() == 168)
+         );
+}
+
+
+void H323EndPoint::PortInfo::Set(unsigned newBase,
+                                 unsigned newMax,
+                                 unsigned range,
+                                 unsigned dflt)
+{
+  if (newBase == 0) {
+    newBase = dflt;
+    newMax = dflt;
+    if (dflt > 0)
+      newMax += range;
+  }
+  else {
+    if (newBase < 1024)
+      newBase = 1024;
+    else if (newBase > 65500)
+      newBase = 65500;
+
+    if (newMax <= newBase)
+      newMax = newBase + range;
+    if (newMax > 65535)
+      newMax = 65535;
+  }
+
+  mutex.Wait();
+
+  current = base = (WORD)newBase;
+  max = (WORD)newMax;
+
+  mutex.Signal();
+}
+
+
+WORD H323EndPoint::PortInfo::GetNext(unsigned increment)
+{
+  PWaitAndSignal m(mutex);
+
+  if (current < base || current > (max-increment))
+    current = base;
+
+  if (current == 0)
+    return 0;
+
+  WORD p = current;
+  current = (WORD)(current + increment);
+  return p;
 }
 
 
 void H323EndPoint::SetTCPPorts(unsigned tcpBase, unsigned tcpMax)
 {
-  if (tcpBase == 0)
-    tcpPortBase = tcpPortMax = 0;
-  else {
-    LimitPorts(tcpBase, tcpMax, 99);
-
-    tcpPortBase = (WORD)tcpBase;
-    tcpPortMax = (WORD)tcpMax;
-  }
+  tcpPorts.Set(tcpBase, tcpMax, 99, 0);
 }
 
 
-void H323EndPoint::SetRtpIpPorts(unsigned udpBase, unsigned udpMax)
+WORD H323EndPoint::GetNextTCPPort()
 {
-  LimitPorts(udpBase, udpMax, 399);
+  return tcpPorts.GetNext(1);
+}
 
-  rtpIpPortBase = (WORD)udpBase;
-  rtpIpPortMax  = (WORD)udpMax;
+
+void H323EndPoint::SetUDPPorts(unsigned udpBase, unsigned udpMax)
+{
+  udpPorts.Set(udpBase, udpMax, 199, 0);
+}
+
+
+WORD H323EndPoint::GetNextUDPPort()
+{
+  return udpPorts.GetNext(1);
+}
+
+
+void H323EndPoint::SetRtpIpPorts(unsigned rtpIpBase, unsigned rtpIpMax)
+{
+  rtpIpPorts.Set((rtpIpBase+1)&0xfffe, rtpIpMax&0xfffe, 999, 5000);
+}
+
+
+WORD H323EndPoint::GetRtpIpPortPair()
+{
+  return rtpIpPorts.GetNext(2);
+}
+
+const PTimeInterval & H323EndPoint::GetNoMediaTimeout() const
+{
+  PWaitAndSignal m(noMediaMutex);
+  
+  return noMediaTimeout; 
+}
+
+BOOL H323EndPoint::SetNoMediaTimeout(PTimeInterval newInterval) 
+{
+  PWaitAndSignal m(noMediaMutex);
+
+  if (newInterval < 0)
+    return FALSE;
+
+  noMediaTimeout = newInterval; 
+  return TRUE; 
 }
 
 
