@@ -27,6 +27,25 @@
  * Contributor(s): ______________________________________.
  *
  * $Log: jitter.cxx,v $
+ * Revision 1.33  2002/11/26 03:00:06  robertj
+ * Added logging to help find logical channel thread stop failures.
+ *
+ * Revision 1.32  2002/11/05 04:06:48  robertj
+ * Better tracing
+ *
+ * Revision 1.31  2002/10/31 00:46:30  robertj
+ * Enhanced jitter buffer system so operates dynamically between minimum and
+ *   maximum values. Altered API to assure app writers note the change!
+ *
+ * Revision 1.30  2002/10/30 05:56:12  craigs
+ * Added more bulletproofing thanks to Alex Kovatch
+ *
+ * Revision 1.29  2002/09/03 07:31:35  robertj
+ * Added buffer reset on excess buffer overruns.
+ *
+ * Revision 1.28  2002/08/05 10:03:47  robertj
+ * Cosmetic changes to normalise the usage of pragma interface/implementation.
+ *
  * Revision 1.27  2002/01/17 07:01:28  robertj
  * Fixed jitter buffer failing to deliver a talk burst shorted than the size of
  *    the buffer, this is particularly noticable with RFC2833.
@@ -114,13 +133,32 @@
  *
  */
 
+#include <ptlib.h>
+
 #ifdef __GNUC__
 #pragma implementation "jitter.h"
 #endif
 
-#include <ptlib.h>
 #include "jitter.h"
 
+
+/*Number of consecutive attempts to add a packet to the jitter buffer while
+  it is full before the system clears the jitter buffer and starts over
+  again. */
+#define MAX_BUFFER_OVERRUNS 20
+
+/**How much time must elapse with consecutive "early" packets before jitter
+   buffer size is reduced and the amount that it is reduced by. */
+#define EARLY_PACKET_PERIOD    10    // RTP timestamp units (milliseconds)
+#define DECREASE_JITTER_PERIOD 16000 // RTP timestamp units (2 seconds)
+#define DECREASE_JITTER_AMOUNT 8     // RTP timestamp units (1 millisecond)
+
+/**Amount to increase jitter buffer size every time a late packet is
+   received. */
+#define INCREASE_JITTER_AMOUNT 16    // RTP timestamp units (2 millisecond)
+
+
+
 #if PTRACING && !defined(NO_ANALYSER)
 
 class RTP_JitterBufferAnalyser : public PObject
@@ -151,7 +189,8 @@ class RTP_JitterBufferAnalyser : public PObject
 /////////////////////////////////////////////////////////////////////////////
 
 RTP_JitterBuffer::RTP_JitterBuffer(RTP_Session & sess,
-                                   unsigned jitterDelay,
+                                   unsigned minJitterDelay,
+                                   unsigned maxJitterDelay,
                                    PINDEX stackSize)
   : PThread(stackSize, NoAutoDeleteThread, HighestPriority, "RTP Jitter:%x"),
     session(sess)
@@ -163,17 +202,23 @@ RTP_JitterBuffer::RTP_JitterBuffer(RTP_Session & sess,
   oldestFrame = newestFrame = currentWriteFrame = NULL;
 
   // Calculate the maximum amount of timestamp units for the jitter buffer
-  maxJitterTime = jitterDelay;
+  minJitterTime = minJitterDelay;
+  maxJitterTime = maxJitterDelay;
+  currentJitterTime = minJitterDelay + (maxJitterDelay - minJitterDelay)/3;
 
   // Calculate number of frames to allocate, we make the assumption that the
   // smallest packet we can possibly get is 5ms long (assuming audio 8kHz unit).
-  bufferSize = jitterDelay/40+1;
+  bufferSize = maxJitterTime/40+1;
 
   // Nothing in the buffer so far
   currentDepth = 0;
   packetsTooLate = 0;
+  bufferOverruns = 0;
+  consecutiveBufferOverruns = 0;
   maxConsecutiveMarkerBits = 10;
   consecutiveMarkerBits = 0;
+  consecutiveEarlyPacketStartTime = 0;
+
   shuttingDown = FALSE;
   preBuffering = TRUE;
 
@@ -191,7 +236,9 @@ RTP_JitterBuffer::RTP_JitterBuffer(RTP_Session & sess,
 
   PTRACE(2, "RTP\tJitter buffer created:"
             " size=" << bufferSize <<
-            " delay=" << maxJitterTime);
+            " delay=" << minJitterTime << '-' << maxJitterTime << '/' << currentJitterTime <<
+            " (" << (currentJitterTime/8) << "ms)"
+            " obj=" << this);
 
 #if PTRACING && !defined(NO_ANALYSER)
   analyser = new RTP_JitterBufferAnalyser;
@@ -206,7 +253,7 @@ RTP_JitterBuffer::RTP_JitterBuffer(RTP_Session & sess,
 
 RTP_JitterBuffer::~RTP_JitterBuffer()
 {
-  PTRACE(3, "RTP\tRemoving jitter buffer");
+  PTRACE(3, "RTP\tRemoving jitter buffer " << this << ' ' << GetThreadName());
 
   shuttingDown = TRUE;
   PAssert(WaitForTermination(10000), "Jitter buffer thread did not terminate");
@@ -232,20 +279,22 @@ RTP_JitterBuffer::~RTP_JitterBuffer()
 
 #if PTRACING && !defined(NO_ANALYSER)
   PTRACE(5, "Jitter buffer analysis: size=" << bufferSize
-         << " time=" << maxJitterTime << '\n' << *analyser);
+         << " time=" << currentJitterTime << '\n' << *analyser);
   delete analyser;
 #endif
 }
 
 
-void RTP_JitterBuffer::SetDelay( unsigned delay)
+void RTP_JitterBuffer::SetDelay(unsigned minJitterDelay, unsigned maxJitterDelay)
 {
-  bufferMutex.Wait();
-
   if (shuttingDown)
     PAssert(WaitForTermination(10000), "Jitter buffer thread did not terminate");
 
-  maxJitterTime = delay;
+  bufferMutex.Wait();
+
+  minJitterTime = minJitterDelay;
+  maxJitterTime = maxJitterDelay;
+  currentJitterTime = minJitterDelay + (maxJitterDelay - minJitterDelay)/3;
 
   PINDEX newBufferSize = maxJitterTime/40+1;
   while (bufferSize < newBufferSize) {
@@ -259,13 +308,18 @@ void RTP_JitterBuffer::SetDelay( unsigned delay)
 
   if (IsTerminated()) {
     packetsTooLate = 0;
+    bufferOverruns = 0;
+    consecutiveBufferOverruns = 0;
     consecutiveMarkerBits = 0;
+    consecutiveEarlyPacketStartTime = 0;
+
     shuttingDown = FALSE;
     preBuffering = TRUE;
 
     PTRACE(2, "RTP\tJitter buffer restarted:"
               " size=" << bufferSize <<
-              " delay=" << maxJitterTime);
+              " delay=" << minJitterTime << '-' << maxJitterTime << '/' << currentJitterTime <<
+              " (" << (currentJitterTime/8) << "ms)");
     Restart();
   }
 
@@ -275,7 +329,7 @@ void RTP_JitterBuffer::SetDelay( unsigned delay)
 
 void RTP_JitterBuffer::Main()
 {
-  PTRACE(3, "RTP\tJitter RTP receive thread started");
+  PTRACE(3, "RTP\tJitter RTP receive thread started: " << this);
 
   bufferMutex.Wait();
 
@@ -290,6 +344,10 @@ void RTP_JitterBuffer::Main()
       freeFrames = freeFrames->next;
       if (freeFrames != NULL)
         freeFrames->prev = NULL;
+      PTRACE_IF(2, consecutiveBufferOverruns > 1,
+                "RTP\tJitter buffer full, threw away "
+                << consecutiveBufferOverruns << " oldest frames");
+      consecutiveBufferOverruns = 0;
     }
     else {
       // We have a full jitter buffer, need a new frame so take the oldest one
@@ -298,8 +356,19 @@ void RTP_JitterBuffer::Main()
       if (oldestFrame != NULL)
         oldestFrame->prev = NULL;
       currentDepth--;
-      packetsTooLate++;
-      PTRACE(4, "RTP\tJitter buffer full, throwing away oldest frame");
+      bufferOverruns++;
+      consecutiveBufferOverruns++;
+      if (consecutiveBufferOverruns > MAX_BUFFER_OVERRUNS) {
+        PTRACE(2, "RTP\tJitter buffer continuously full, throwing away entire buffer.");
+        freeFrames = oldestFrame;
+        oldestFrame = newestFrame = NULL;
+        preBuffering = TRUE;
+      }
+      else {
+        PTRACE_IF(2, consecutiveBufferOverruns == 1,
+                  "RTP\tJitter buffer full, throwing away oldest frame ("
+                  << currentReadFrame->GetTimestamp() << ')');
+      }
     }
 
     currentReadFrame->next = NULL;
@@ -314,6 +383,8 @@ void RTP_JitterBuffer::Main()
       return;
     }
 
+    currentReadFrame->tick = PTimer::Tick();
+
     if (currentReadFrame->GetMarker()) {
       // See if remote appears to be setting marker bit on EVERY packet.
       consecutiveMarkerBits++;
@@ -396,6 +467,8 @@ BOOL RTP_JitterBuffer::ReadData(DWORD timestamp, RTP_DataFrame & frame)
   // Default response is an empty frame, ie silence
   frame.SetPayloadSize(0);
 
+  PWaitAndSignal mutex(bufferMutex);
+
   /*Get the next frame to write to the codec. Takes it from the oldest
     position in the queue, if it is time to do so, and parks it in the
     special member so can unlock the mutex while the writer thread has its
@@ -413,8 +486,6 @@ BOOL RTP_JitterBuffer::ReadData(DWORD timestamp, RTP_DataFrame & frame)
     return TRUE;
   }
 
-  PWaitAndSignal mutex(bufferMutex);
-
   /* See if time for this packet, if our oldest frame is older than the
      required age, then use it. If it is not time yet, make sure that the
      writer thread isn't falling behind (not enough MIPS). If the time
@@ -429,8 +500,8 @@ BOOL RTP_JitterBuffer::ReadData(DWORD timestamp, RTP_DataFrame & frame)
   if (preBuffering) {
     // Check for requesting something that already exceeds the maximum time,
     // or have filled the jitter buffer, not filling if this is so
-    if ((timestamp - oldestTimestamp) < maxJitterTime &&
-        (newestTimestamp - oldestTimestamp) < maxJitterTime/2) {
+    if ((timestamp - oldestTimestamp) < currentJitterTime &&
+        (newestTimestamp - oldestTimestamp) < currentJitterTime/2) {
       // Are filling the buffer, don't return anything yet
 #if PTRACING && !defined(NO_ANALYSER)
       analyser->Out(oldestTimestamp, currentDepth, "PreBuf");
@@ -441,7 +512,7 @@ BOOL RTP_JitterBuffer::ReadData(DWORD timestamp, RTP_DataFrame & frame)
     preBuffering = FALSE;
   }
 
-  if (timestamp < oldestTimestamp && timestamp > (newestTimestamp - maxJitterTime)) {
+  if (timestamp < oldestTimestamp && timestamp > (newestTimestamp - currentJitterTime)) {
     // It is not yet time for something in the buffer
 #if PTRACING && !defined(NO_ANALYSER)
     analyser->Out(oldestTimestamp, currentDepth, "Wait");
@@ -464,9 +535,20 @@ BOOL RTP_JitterBuffer::ReadData(DWORD timestamp, RTP_DataFrame & frame)
     oldestFrame->prev = NULL;
 
     // See if exceeded maximum jitter buffer time delay, waste them if so
-    while ((newestTimestamp - oldestFrame->GetTimestamp()) > maxJitterTime) {
+    while ((newestTimestamp - oldestFrame->GetTimestamp()) > currentJitterTime) {
       PTRACE(4, "RTP\tJitter buffer oldest packet ("
-             << oldestFrame->GetTimestamp() << ") too late, throwing away");
+             << oldestFrame->GetTimestamp() << " < "
+             << (newestTimestamp - maxJitterTime)
+             << ") too late, throwing away");
+
+      packetsTooLate++;
+      consecutiveEarlyPacketStartTime = newestTimestamp;
+
+      if (currentJitterTime < maxJitterTime) {
+        currentJitterTime += INCREASE_JITTER_AMOUNT;
+        PTRACE(3, "RTP\tJitter buffer size increased to "
+               << currentJitterTime << " (" << (currentJitterTime/8) << "ms)");
+      }
 
       // Throw away the oldest entry
       Entry * wastedFrame = oldestFrame;
@@ -479,11 +561,21 @@ BOOL RTP_JitterBuffer::ReadData(DWORD timestamp, RTP_DataFrame & frame)
       if (freeFrames != NULL)
         freeFrames->prev = wastedFrame;
       freeFrames = wastedFrame;
-
-      packetsTooLate++;
     }
   }
 
+  PTimeInterval delay = PTimer::Tick() - currentWriteFrame->tick;
+  if (consecutiveEarlyPacketStartTime == 0 || delay < EARLY_PACKET_PERIOD)
+    consecutiveEarlyPacketStartTime = newestTimestamp;
+
+  if (currentJitterTime > minJitterTime &&
+        (newestTimestamp - consecutiveEarlyPacketStartTime) > DECREASE_JITTER_PERIOD) {
+    consecutiveEarlyPacketStartTime = newestTimestamp;
+    currentJitterTime -= DECREASE_JITTER_AMOUNT;
+    PTRACE(3, "RTP\tJitter buffer size decreased to "
+           << currentJitterTime << " (" << (currentJitterTime/8) << "ms)");
+  }
+
   frame = *currentWriteFrame;
   return TRUE;
 }
